//=============================================================================================================================
//
// Copyright (c) 2015-2019 VisionStar Information Technology (Shanghai) Co., Ltd. All Rights Reserved.
// EasyAR is the registered trademark or trademark of VisionStar Information Technology (Shanghai) Co., Ltd in China
// and other countries for the augmented reality technology developed by VisionStar Information Technology (Shanghai) Co., Ltd.
//
//=============================================================================================================================

#import "OpenGLView.h"
#import "AppDelegate.h"

#include "helloar.hpp"

#include <easyar/types.hpp>

@implementation OpenGLView {
    bool initialized;
}

- (id)initWithFrame:(CGRect)frame
{
    createScheduler();
    self = [super initWithFrame:frame];
    self->initialized = false;
    self.context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES2];
    self.drawableColorFormat = GLKViewDrawableColorFormatRGBA8888;
    self.drawableDepthFormat = GLKViewDrawableDepthFormat24;
    self.drawableStencilFormat = GLKViewDrawableStencilFormat8;
    [self bindDrawable];
    initialize();
    return self;
}

- (void)dealloc
{
    finalize();
}

- (void)start
{
    easyar::CameraDevice::requestPermissions(getScheduler(), [](easyar::PermissionStatus status, std::string value) {
        switch (status) {
            case easyar::PermissionStatus::Denied:
                NSLog(@"camera permission denied");
                break;
            case easyar::PermissionStatus::Granted:
                start();
                break;
            case easyar::PermissionStatus::Error:
                NSLog(@"camera permission error");
                break;
            default:
                break;
        }
    });
}

- (void)stop
{
    stop();
}

- (void)drawRect:(CGRect)rect
{
    float scale;
    if ([[UIDevice currentDevice].systemVersion floatValue] >= 8.0f) {
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wpartial-availability"
        scale = [[UIScreen mainScreen] nativeScale];
#pragma clang diagnostic pop
    } else {
        scale = [[UIScreen mainScreen] scale];
    }

    render(rect.size.width * scale, rect.size.height * scale, _screenRotation);
}

- (void)setOrientation:(UIInterfaceOrientation)orientation
{
    switch (orientation)
    {
        case UIInterfaceOrientationPortrait:
            _screenRotation = 0;
            break;
        case UIInterfaceOrientationLandscapeRight:
            _screenRotation = 90;
            break;
        case UIInterfaceOrientationPortraitUpsideDown:
            _screenRotation = 180;
            break;
        case UIInterfaceOrientationLandscapeLeft:
            _screenRotation = 270;
            break;
        default:
            break;
    }
}

@end
