//=============================================================================================================================
//
// Copyright (c) 2015-2019 VisionStar Information Technology (Shanghai) Co., Ltd. All Rights Reserved.
// EasyAR is the registered trademark or trademark of VisionStar Information Technology (Shanghai) Co., Ltd in China
// and other countries for the augmented reality technology developed by VisionStar Information Technology (Shanghai) Co., Ltd.
//
//=============================================================================================================================

#import "OpenGLView.h"
#import "AppDelegate.h"

#import "helloar.h"

#import <easyar/engine.oc.h>
#import <easyar/camera.oc.h>
#import <easyar/callbackscheduler.oc.h>

@interface OpenGLView ()

- (void)onStartStopClicked:(id)sender;

@end

@implementation OpenGLView {
    BOOL started;
    UIButton * startStopButton;
}

- (id)initWithFrame:(CGRect)frame
{
    createScheduler();
    self = [super initWithFrame:frame];
    self.context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES2];
    self.drawableColorFormat = GLKViewDrawableColorFormatRGBA8888;
    self.drawableDepthFormat = GLKViewDrawableDepthFormat24;
    self.drawableStencilFormat = GLKViewDrawableStencilFormat8;
    [self bindDrawable];
    initialize();

    startStopButton = [[UIButton alloc] init];
    [startStopButton setTitle:@"Start" forState:UIControlStateNormal];
    startStopButton.contentEdgeInsets = UIEdgeInsetsMake(10, 10, 10, 10);
    [startStopButton sizeToFit];
    [startStopButton setBackgroundColor:[UIColor grayColor]];
    [self addSubview:startStopButton];
    startStopButton.translatesAutoresizingMaskIntoConstraints = NO;
    if ([[UIDevice currentDevice].systemVersion floatValue] >= 9.0f) {
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wpartial-availability"
        [startStopButton.centerXAnchor constraintEqualToAnchor: self.layoutMarginsGuide.centerXAnchor].active = YES;
        [startStopButton.bottomAnchor constraintEqualToAnchor: self.layoutMarginsGuide.bottomAnchor].active = YES;
#pragma clang diagnostic pop
    }
    
    [startStopButton addTarget:self action:@selector(onStartStopClicked:) forControlEvents:UIControlEventTouchUpInside];
    
    return self;
}

- (void)onStartStopClicked:(id)sender
{
    UIButton * startStopButton = (UIButton *)sender;
    if (started) {
        [startStopButton setTitle:@"Start" forState:UIControlStateNormal];
        stopTracker();
    } else {
        [startStopButton setTitle:@"Stop" forState:UIControlStateNormal];
        startTracker();
    }
    started = !started;
}

- (void)dealloc
{
    finalize();
}

- (void)start
{
    [easyar_CameraDevice requestPermissions:getScheduler() permissionCallback:^(easyar_PermissionStatus status, NSString *value) {
        switch (status) {
            case easyar_PermissionStatus_Denied:
                NSLog(@"camera permission denied");
                break;
            case easyar_PermissionStatus_Granted:
                start();
                break;
            case easyar_PermissionStatus_Error:
                NSLog(@"camera permission error");
                break;
            default:
                break;
        }
    }];
}

- (void)stop
{
    stop();
}

- (void)resize:(CGRect)frame orientation:(UIInterfaceOrientation)orientation
{
    if ([[UIDevice currentDevice].systemVersion floatValue] >= 9.0f) {
    } else {
        if (startStopButton != nil) {
            startStopButton.frame = CGRectMake(frame.size.width / 2 - startStopButton.frame.size.width / 2, frame.size.height - startStopButton.frame.size.height, startStopButton.frame.size.width, startStopButton.frame.size.height);
        }
    }
}

- (void)drawRect:(CGRect)rect
{
    float scale;
    if ([[UIDevice currentDevice].systemVersion floatValue] >= 8.0f) {
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wpartial-availability"
        scale = [[UIScreen mainScreen] nativeScale];
#pragma clang diagnostic pop
    } else {
        scale = [[UIScreen mainScreen] scale];
    }

    render(rect.size.width * scale, rect.size.height * scale, _screenRotation);
}

- (void)setOrientation:(UIInterfaceOrientation)orientation
{
    switch (orientation)
    {
        case UIInterfaceOrientationPortrait:
            _screenRotation = 0;
            break;
        case UIInterfaceOrientationLandscapeRight:
            _screenRotation = 90;
            break;
        case UIInterfaceOrientationPortraitUpsideDown:
            _screenRotation = 180;
            break;
        case UIInterfaceOrientationLandscapeLeft:
            _screenRotation = 270;
            break;
        default:
            break;
    }
}

@end
