﻿//=============================================================================================================================
//
// EasyAR 3.0.0-final-r2c8ac46a
// Copyright (c) 2015-2019 VisionStar Information Technology (Shanghai) Co., Ltd. All Rights Reserved.
// EasyAR is the registered trademark or trademark of VisionStar Information Technology (Shanghai) Co., Ltd in China
// and other countries for the augmented reality technology developed by VisionStar Information Technology (Shanghai) Co., Ltd.
//
//=============================================================================================================================

import Foundation

public class RefBase {
    fileprivate let cdata_: OpaquePointer
    fileprivate let deleter_: (OpaquePointer) -> Void

    fileprivate init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        cdata_ = cdata
        deleter_ = deleter
    }

    deinit {
        deleter_(cdata_)
    }
}

let TypeNameToConstructor: [String : (OpaquePointer) -> RefBase] = [
    "ObjectTargetParameters": { cdata in ObjectTargetParameters(cdata: cdata, deleter: easyar_ObjectTargetParameters__dtor) },
    "ObjectTarget": { cdata in ObjectTarget(cdata: cdata, deleter: easyar_ObjectTarget__dtor) },
    "ObjectTrackerResult": { cdata in ObjectTrackerResult(cdata: cdata, deleter: easyar_ObjectTrackerResult__dtor) },
    "ObjectTracker": { cdata in ObjectTracker(cdata: cdata, deleter: easyar_ObjectTracker__dtor) },
    "CADTarget": { cdata in CADTarget(cdata: cdata, deleter: easyar_CADTarget__dtor) },
    "CADTrackerResult": { cdata in CADTrackerResult(cdata: cdata, deleter: easyar_CADTrackerResult__dtor) },
    "CADTracker": { cdata in CADTracker(cdata: cdata, deleter: easyar_CADTracker__dtor) },
    "CloudRecognizer": { cdata in CloudRecognizer(cdata: cdata, deleter: easyar_CloudRecognizer__dtor) },
    "DenseSpatialMap": { cdata in DenseSpatialMap(cdata: cdata, deleter: easyar_DenseSpatialMap__dtor) },
    "SceneMesh": { cdata in SceneMesh(cdata: cdata, deleter: easyar_SceneMesh__dtor) },
    "SurfaceTrackerResult": { cdata in SurfaceTrackerResult(cdata: cdata, deleter: easyar_SurfaceTrackerResult__dtor) },
    "SurfaceTracker": { cdata in SurfaceTracker(cdata: cdata, deleter: easyar_SurfaceTracker__dtor) },
    "MotionTrackerResult": { cdata in MotionTrackerResult(cdata: cdata, deleter: easyar_MotionTrackerResult__dtor) },
    "MotionTracker": { cdata in MotionTracker(cdata: cdata, deleter: easyar_MotionTracker__dtor) },
    "MotionTrackerOutputFrameToInputFrameAdapter": { cdata in MotionTrackerOutputFrameToInputFrameAdapter(cdata: cdata, deleter: easyar_MotionTrackerOutputFrameToInputFrameAdapter__dtor) },
    "ImageTargetParameters": { cdata in ImageTargetParameters(cdata: cdata, deleter: easyar_ImageTargetParameters__dtor) },
    "ImageTarget": { cdata in ImageTarget(cdata: cdata, deleter: easyar_ImageTarget__dtor) },
    "ImageTrackerResult": { cdata in ImageTrackerResult(cdata: cdata, deleter: easyar_ImageTrackerResult__dtor) },
    "ImageTracker": { cdata in ImageTracker(cdata: cdata, deleter: easyar_ImageTracker__dtor) },
    "Recorder": { cdata in Recorder(cdata: cdata, deleter: easyar_Recorder__dtor) },
    "RecorderConfiguration": { cdata in RecorderConfiguration(cdata: cdata, deleter: easyar_RecorderConfiguration__dtor) },
    "SparseSpatialMapResult": { cdata in SparseSpatialMapResult(cdata: cdata, deleter: easyar_SparseSpatialMapResult__dtor) },
    "PlaneData": { cdata in PlaneData(cdata: cdata, deleter: easyar_PlaneData__dtor) },
    "SparseSpatialMapConfig": { cdata in SparseSpatialMapConfig(cdata: cdata, deleter: easyar_SparseSpatialMapConfig__dtor) },
    "SparseSpatialMap": { cdata in SparseSpatialMap(cdata: cdata, deleter: easyar_SparseSpatialMap__dtor) },
    "CallbackScheduler": { cdata in CallbackScheduler(cdata: cdata, deleter: easyar_CallbackScheduler__dtor) },
    "DelayedCallbackScheduler": { cdata in DelayedCallbackScheduler(cdata: cdata, deleter: easyar_DelayedCallbackScheduler__dtor) },
    "ImmediateCallbackScheduler": { cdata in ImmediateCallbackScheduler(cdata: cdata, deleter: easyar_ImmediateCallbackScheduler__dtor) },
    "CameraDevice": { cdata in CameraDevice(cdata: cdata, deleter: easyar_CameraDevice__dtor) },
    "SignalSink": { cdata in SignalSink(cdata: cdata, deleter: easyar_SignalSink__dtor) },
    "SignalSource": { cdata in SignalSource(cdata: cdata, deleter: easyar_SignalSource__dtor) },
    "InputFrameSink": { cdata in InputFrameSink(cdata: cdata, deleter: easyar_InputFrameSink__dtor) },
    "InputFrameSource": { cdata in InputFrameSource(cdata: cdata, deleter: easyar_InputFrameSource__dtor) },
    "OutputFrameSink": { cdata in OutputFrameSink(cdata: cdata, deleter: easyar_OutputFrameSink__dtor) },
    "OutputFrameSource": { cdata in OutputFrameSource(cdata: cdata, deleter: easyar_OutputFrameSource__dtor) },
    "FeedbackFrameSink": { cdata in FeedbackFrameSink(cdata: cdata, deleter: easyar_FeedbackFrameSink__dtor) },
    "FeedbackFrameSource": { cdata in FeedbackFrameSource(cdata: cdata, deleter: easyar_FeedbackFrameSource__dtor) },
    "InputFrameFork": { cdata in InputFrameFork(cdata: cdata, deleter: easyar_InputFrameFork__dtor) },
    "OutputFrameFork": { cdata in OutputFrameFork(cdata: cdata, deleter: easyar_OutputFrameFork__dtor) },
    "OutputFrameJoin": { cdata in OutputFrameJoin(cdata: cdata, deleter: easyar_OutputFrameJoin__dtor) },
    "FeedbackFrameFork": { cdata in FeedbackFrameFork(cdata: cdata, deleter: easyar_FeedbackFrameFork__dtor) },
    "InputFrameThrottler": { cdata in InputFrameThrottler(cdata: cdata, deleter: easyar_InputFrameThrottler__dtor) },
    "OutputFrameBuffer": { cdata in OutputFrameBuffer(cdata: cdata, deleter: easyar_OutputFrameBuffer__dtor) },
    "InputFrameToOutputFrameAdapter": { cdata in InputFrameToOutputFrameAdapter(cdata: cdata, deleter: easyar_InputFrameToOutputFrameAdapter__dtor) },
    "InputFrameToFeedbackFrameAdapter": { cdata in InputFrameToFeedbackFrameAdapter(cdata: cdata, deleter: easyar_InputFrameToFeedbackFrameAdapter__dtor) },
    "InputFrame": { cdata in InputFrame(cdata: cdata, deleter: easyar_InputFrame__dtor) },
    "FrameFilterResult": { cdata in FrameFilterResult(cdata: cdata, deleter: easyar_FrameFilterResult__dtor) },
    "OutputFrame": { cdata in OutputFrame(cdata: cdata, deleter: easyar_OutputFrame__dtor) },
    "FeedbackFrame": { cdata in FeedbackFrame(cdata: cdata, deleter: easyar_FeedbackFrame__dtor) },
    "Target": { cdata in Target(cdata: cdata, deleter: easyar_Target__dtor) },
    "TargetInstance": { cdata in TargetInstance(cdata: cdata, deleter: easyar_TargetInstance__dtor) },
    "TargetTrackerResult": { cdata in TargetTrackerResult(cdata: cdata, deleter: easyar_TargetTrackerResult__dtor) },
    "TextureId": { cdata in TextureId(cdata: cdata, deleter: easyar_TextureId__dtor) },
    "VideoPlayer": { cdata in VideoPlayer(cdata: cdata, deleter: easyar_VideoPlayer__dtor) },
    "Buffer": { cdata in Buffer(cdata: cdata, deleter: easyar_Buffer__dtor) },
    "BufferDictionary": { cdata in BufferDictionary(cdata: cdata, deleter: easyar_BufferDictionary__dtor) },
    "BufferPool": { cdata in BufferPool(cdata: cdata, deleter: easyar_BufferPool__dtor) },
    "CameraParameters": { cdata in CameraParameters(cdata: cdata, deleter: easyar_CameraParameters__dtor) },
    "Image": { cdata in Image(cdata: cdata, deleter: easyar_Image__dtor) },
]

fileprivate class OpaquePointerContainer {
    public let ptr_: OpaquePointer
    public let deleter_: (OpaquePointer) -> Void
    public init(_ ptr: OpaquePointer, _ deleter: @escaping (OpaquePointer) -> Void) {
        ptr_ = ptr
        deleter_ = deleter
    }
    deinit {
        deleter_(ptr_)
    }
}

fileprivate func defaultValue<T>() -> T {
    let ptr = UnsafeMutablePointer<T>.allocate(capacity: 1)
    defer {
        ptr.deallocate()
    }
    let retval = ptr.pointee
    return retval;
}

fileprivate func Optional_retain(_ ptr: OpaquePointer?, _ retain: @convention(c) (OpaquePointer, UnsafeMutablePointer<OpaquePointer>) -> Void) -> OpaquePointer? {
    if let p = ptr {
        let pptr = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            pptr.deallocate()
        }
        retain(p, &pptr.pointee)
        return pptr.pointee
    } else {
        return nil
    }
}

fileprivate func String_to_c(_ s: String) -> OpaquePointerContainer {
    let pptr = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
    defer {
        pptr.deallocate()
    }
    s.utf8CString.withUnsafeBufferPointer { p in
        easyar_String_from_utf8(p.baseAddress, p.baseAddress?.advanced(by: p.count - 1), pptr)
    }
    let ptr = pptr.pointee
    return OpaquePointerContainer(ptr, { p in easyar_String__dtor(p) })
}

fileprivate func String_from_c(_ ptr: OpaquePointerContainer) -> String {
    let begin = easyar_String_begin(ptr.ptr_)
    let count = begin == nil ? 0 : begin!.distance(to: easyar_String_end(ptr.ptr_)!)
    let arr = [UInt8](UnsafeBufferPointer<UInt8>(start: UnsafePointer<UInt8>(OpaquePointer(begin)), count: count))
    return String(bytes: arr, encoding: String.Encoding.utf8)!
}

fileprivate func Object_from_c<T>(_ ptr: OpaquePointer, _ typeNameGetter: (OpaquePointer) -> UnsafePointer<CChar>) -> T {
    let typeName = String(cString: typeNameGetter(ptr))
    if let ctor = TypeNameToConstructor[typeName] {
        if let o = ctor(ptr) as? T {
            return o
        } else {
            fatalError("ValueNotOfType")
        }
    } else {
        fatalError("ConstructorNotExistForType")
    }
}

fileprivate func BlockInfo_to_c(_ r: BlockInfo) -> easyar_BlockInfo {
    return easyar_BlockInfo(x_: r.x_, y_: r.y_, z_: r.z_, numOfVertex_: r.numOfVertex_, startPointOfVertex_: r.startPointOfVertex_, numOfIndex_: r.numOfIndex_, startPointOfIndex_: r.startPointOfIndex_, age_: r.age_)
}
fileprivate func BlockInfo_from_c(_ r: easyar_BlockInfo) -> BlockInfo {
    return BlockInfo(r.x_, r.y_, r.z_, r.numOfVertex_, r.startPointOfVertex_, r.numOfIndex_, r.startPointOfIndex_, r.age_)
}

fileprivate func Matrix44F_to_c(_ r: Matrix44F) -> easyar_Matrix44F {
    return easyar_Matrix44F(data: (r.data.0, r.data.1, r.data.2, r.data.3, r.data.4, r.data.5, r.data.6, r.data.7, r.data.8, r.data.9, r.data.10, r.data.11, r.data.12, r.data.13, r.data.14, r.data.15))
}
fileprivate func Matrix44F_from_c(_ r: easyar_Matrix44F) -> Matrix44F {
    return Matrix44F(r.data.0, r.data.1, r.data.2, r.data.3, r.data.4, r.data.5, r.data.6, r.data.7, r.data.8, r.data.9, r.data.10, r.data.11, r.data.12, r.data.13, r.data.14, r.data.15)
}

fileprivate func Matrix33F_to_c(_ r: Matrix33F) -> easyar_Matrix33F {
    return easyar_Matrix33F(data: (r.data.0, r.data.1, r.data.2, r.data.3, r.data.4, r.data.5, r.data.6, r.data.7, r.data.8))
}
fileprivate func Matrix33F_from_c(_ r: easyar_Matrix33F) -> Matrix33F {
    return Matrix33F(r.data.0, r.data.1, r.data.2, r.data.3, r.data.4, r.data.5, r.data.6, r.data.7, r.data.8)
}

fileprivate func Vec4F_to_c(_ r: Vec4F) -> easyar_Vec4F {
    return easyar_Vec4F(data: (r.data.0, r.data.1, r.data.2, r.data.3))
}
fileprivate func Vec4F_from_c(_ r: easyar_Vec4F) -> Vec4F {
    return Vec4F(r.data.0, r.data.1, r.data.2, r.data.3)
}

fileprivate func Vec3F_to_c(_ r: Vec3F) -> easyar_Vec3F {
    return easyar_Vec3F(data: (r.data.0, r.data.1, r.data.2))
}
fileprivate func Vec3F_from_c(_ r: easyar_Vec3F) -> Vec3F {
    return Vec3F(r.data.0, r.data.1, r.data.2)
}

fileprivate func Vec2F_to_c(_ r: Vec2F) -> easyar_Vec2F {
    return easyar_Vec2F(data: (r.data.0, r.data.1))
}
fileprivate func Vec2F_from_c(_ r: easyar_Vec2F) -> Vec2F {
    return Vec2F(r.data.0, r.data.1)
}

fileprivate func Vec4I_to_c(_ r: Vec4I) -> easyar_Vec4I {
    return easyar_Vec4I(data: (r.data.0, r.data.1, r.data.2, r.data.3))
}
fileprivate func Vec4I_from_c(_ r: easyar_Vec4I) -> Vec4I {
    return Vec4I(r.data.0, r.data.1, r.data.2, r.data.3)
}

fileprivate func Vec2I_to_c(_ r: Vec2I) -> easyar_Vec2I {
    return easyar_Vec2I(data: (r.data.0, r.data.1))
}
fileprivate func Vec2I_from_c(_ r: easyar_Vec2I) -> Vec2I {
    return Vec2I(r.data.0, r.data.1)
}

fileprivate func FunctorOfVoid_func(_ state: OpaquePointer, _ _exception: UnsafeMutablePointer<OpaquePointer?>) -> Void {
    _exception.pointee = nil
    let f = UnsafeMutablePointer<() -> Void>(state).pointee
    f()
}
fileprivate func FunctorOfVoid_to_c(_ f: @escaping () -> Void) -> easyar_FunctorOfVoid {
    let s = UnsafeMutablePointer<() -> Void>.allocate(capacity: 1)
    s.initialize(to: f)
    return easyar_FunctorOfVoid(_state: OpaquePointer(s), _func: FunctorOfVoid_func, _destroy: { p in UnsafeMutablePointer<() -> Void>(p).deinitialize(count: 1).deallocate() })
}

fileprivate func ListOfObjectTarget_to_c(_ l: [ObjectTarget]) -> OpaquePointerContainer {
    let arr = l.map({ e in e.cdata_ })
    let pptr = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
    defer {
        pptr.deallocate()
    }
    arr.withUnsafeBufferPointer { bp in
        easyar_ListOfObjectTarget__ctor(UnsafePointer<OpaquePointer>(bp.baseAddress), UnsafePointer<OpaquePointer>(bp.baseAddress?.advanced(by: bp.count)), pptr)
    }
    return OpaquePointerContainer(pptr.pointee, { p in easyar_ListOfObjectTarget__dtor(p) })
}
fileprivate func ListOfObjectTarget_from_c(_ l: OpaquePointerContainer) -> [ObjectTarget] {
    let size = Int(easyar_ListOfObjectTarget_size(l.ptr_))
    var values = [ObjectTarget]()
    values.reserveCapacity(size)
    for k in 0..<size {
        var v = easyar_ListOfObjectTarget_at(l.ptr_, Int32(k))
        easyar_ObjectTarget__retain(v, &v)
        values.append((Object_from_c(Optional.some(v)!, easyar_ObjectTarget__typeName) as ObjectTarget))
    }
    return values
}

fileprivate func ListOfVec3F_to_c(_ l: [Vec3F]) -> OpaquePointerContainer {
    let arr = l.map({ e in Vec3F_to_c(e) })
    let pptr = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
    defer {
        pptr.deallocate()
    }
    arr.withUnsafeBufferPointer { bp in
        easyar_ListOfVec3F__ctor(UnsafePointer<easyar_Vec3F>(bp.baseAddress), UnsafePointer<easyar_Vec3F>(bp.baseAddress?.advanced(by: bp.count)), pptr)
    }
    return OpaquePointerContainer(pptr.pointee, { p in easyar_ListOfVec3F__dtor(p) })
}
fileprivate func ListOfVec3F_from_c(_ l: OpaquePointerContainer) -> [Vec3F] {
    let size = Int(easyar_ListOfVec3F_size(l.ptr_))
    var values = [Vec3F]()
    values.reserveCapacity(size)
    for k in 0..<size {
        let v = easyar_ListOfVec3F_at(l.ptr_, Int32(k))
        values.append(Vec3F_from_c(v))
    }
    return values
}

fileprivate func ListOfTargetInstance_to_c(_ l: [TargetInstance]) -> OpaquePointerContainer {
    let arr = l.map({ e in e.cdata_ })
    let pptr = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
    defer {
        pptr.deallocate()
    }
    arr.withUnsafeBufferPointer { bp in
        easyar_ListOfTargetInstance__ctor(UnsafePointer<OpaquePointer>(bp.baseAddress), UnsafePointer<OpaquePointer>(bp.baseAddress?.advanced(by: bp.count)), pptr)
    }
    return OpaquePointerContainer(pptr.pointee, { p in easyar_ListOfTargetInstance__dtor(p) })
}
fileprivate func ListOfTargetInstance_from_c(_ l: OpaquePointerContainer) -> [TargetInstance] {
    let size = Int(easyar_ListOfTargetInstance_size(l.ptr_))
    var values = [TargetInstance]()
    values.reserveCapacity(size)
    for k in 0..<size {
        var v = easyar_ListOfTargetInstance_at(l.ptr_, Int32(k))
        easyar_TargetInstance__retain(v, &v)
        values.append((Object_from_c(Optional.some(v)!, easyar_TargetInstance__typeName) as TargetInstance))
    }
    return values
}

fileprivate func ListOfOptionalOfFrameFilterResult_to_c(_ l: [FrameFilterResult?]) -> OpaquePointerContainer {
    let arr = l.map({ e in (e.map({(p) in easyar_OptionalOfFrameFilterResult(has_value: true, value: p.cdata_)}) ?? easyar_OptionalOfFrameFilterResult(has_value: false, value: defaultValue())) })
    let pptr = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
    defer {
        pptr.deallocate()
    }
    arr.withUnsafeBufferPointer { bp in
        easyar_ListOfOptionalOfFrameFilterResult__ctor(UnsafePointer<easyar_OptionalOfFrameFilterResult>(bp.baseAddress), UnsafePointer<easyar_OptionalOfFrameFilterResult>(bp.baseAddress?.advanced(by: bp.count)), pptr)
    }
    return OpaquePointerContainer(pptr.pointee, { p in easyar_ListOfOptionalOfFrameFilterResult__dtor(p) })
}
fileprivate func ListOfOptionalOfFrameFilterResult_from_c(_ l: OpaquePointerContainer) -> [FrameFilterResult?] {
    let size = Int(easyar_ListOfOptionalOfFrameFilterResult_size(l.ptr_))
    var values = [FrameFilterResult?]()
    values.reserveCapacity(size)
    for k in 0..<size {
        var v = easyar_ListOfOptionalOfFrameFilterResult_at(l.ptr_, Int32(k))
        v.value = Optional_retain(v.value, easyar_FrameFilterResult__retain)
        values.append((v.has_value ? (Object_from_c(Optional.some(v.value!)!, easyar_FrameFilterResult__typeName) as FrameFilterResult) : nil))
    }
    return values
}

fileprivate func FunctorOfVoidFromOutputFrame_func(_ state: OpaquePointer, _ arg0: OpaquePointer, _ _exception: UnsafeMutablePointer<OpaquePointer?>) -> Void {
    _exception.pointee = nil
    var varg0 = arg0
    easyar_OutputFrame__retain(varg0, &varg0)
    let sarg0 = (Object_from_c(Optional.some(varg0)!, easyar_OutputFrame__typeName) as OutputFrame)
    let f = UnsafeMutablePointer<(OutputFrame) -> Void>(state).pointee
    f(sarg0)
}
fileprivate func FunctorOfVoidFromOutputFrame_to_c(_ f: @escaping (OutputFrame) -> Void) -> easyar_FunctorOfVoidFromOutputFrame {
    let s = UnsafeMutablePointer<(OutputFrame) -> Void>.allocate(capacity: 1)
    s.initialize(to: f)
    return easyar_FunctorOfVoidFromOutputFrame(_state: OpaquePointer(s), _func: FunctorOfVoidFromOutputFrame_func, _destroy: { p in UnsafeMutablePointer<(OutputFrame) -> Void>(p).deinitialize(count: 1).deallocate() })
}

fileprivate func FunctorOfVoidFromTargetAndBool_func(_ state: OpaquePointer, _ arg0: OpaquePointer, _ arg1: Bool, _ _exception: UnsafeMutablePointer<OpaquePointer?>) -> Void {
    _exception.pointee = nil
    var varg0 = arg0
    easyar_Target__retain(varg0, &varg0)
    let sarg0 = (Object_from_c(Optional.some(varg0)!, easyar_Target__typeName) as Target)
    let sarg1 = arg1
    let f = UnsafeMutablePointer<(Target, Bool) -> Void>(state).pointee
    f(sarg0, sarg1)
}
fileprivate func FunctorOfVoidFromTargetAndBool_to_c(_ f: @escaping (Target, Bool) -> Void) -> easyar_FunctorOfVoidFromTargetAndBool {
    let s = UnsafeMutablePointer<(Target, Bool) -> Void>.allocate(capacity: 1)
    s.initialize(to: f)
    return easyar_FunctorOfVoidFromTargetAndBool(_state: OpaquePointer(s), _func: FunctorOfVoidFromTargetAndBool_func, _destroy: { p in UnsafeMutablePointer<(Target, Bool) -> Void>(p).deinitialize(count: 1).deallocate() })
}

fileprivate func ListOfTarget_to_c(_ l: [Target]) -> OpaquePointerContainer {
    let arr = l.map({ e in e.cdata_ })
    let pptr = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
    defer {
        pptr.deallocate()
    }
    arr.withUnsafeBufferPointer { bp in
        easyar_ListOfTarget__ctor(UnsafePointer<OpaquePointer>(bp.baseAddress), UnsafePointer<OpaquePointer>(bp.baseAddress?.advanced(by: bp.count)), pptr)
    }
    return OpaquePointerContainer(pptr.pointee, { p in easyar_ListOfTarget__dtor(p) })
}
fileprivate func ListOfTarget_from_c(_ l: OpaquePointerContainer) -> [Target] {
    let size = Int(easyar_ListOfTarget_size(l.ptr_))
    var values = [Target]()
    values.reserveCapacity(size)
    for k in 0..<size {
        var v = easyar_ListOfTarget_at(l.ptr_, Int32(k))
        easyar_Target__retain(v, &v)
        values.append((Object_from_c(Optional.some(v)!, easyar_Target__typeName) as Target))
    }
    return values
}

fileprivate func ListOfCADTarget_to_c(_ l: [CADTarget]) -> OpaquePointerContainer {
    let arr = l.map({ e in e.cdata_ })
    let pptr = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
    defer {
        pptr.deallocate()
    }
    arr.withUnsafeBufferPointer { bp in
        easyar_ListOfCADTarget__ctor(UnsafePointer<OpaquePointer>(bp.baseAddress), UnsafePointer<OpaquePointer>(bp.baseAddress?.advanced(by: bp.count)), pptr)
    }
    return OpaquePointerContainer(pptr.pointee, { p in easyar_ListOfCADTarget__dtor(p) })
}
fileprivate func ListOfCADTarget_from_c(_ l: OpaquePointerContainer) -> [CADTarget] {
    let size = Int(easyar_ListOfCADTarget_size(l.ptr_))
    var values = [CADTarget]()
    values.reserveCapacity(size)
    for k in 0..<size {
        var v = easyar_ListOfCADTarget_at(l.ptr_, Int32(k))
        easyar_CADTarget__retain(v, &v)
        values.append((Object_from_c(Optional.some(v)!, easyar_CADTarget__typeName) as CADTarget))
    }
    return values
}

fileprivate func FunctorOfVoidFromCloudStatusAndListOfTarget_func(_ state: OpaquePointer, _ arg0: Int32, _ arg1: OpaquePointer, _ _exception: UnsafeMutablePointer<OpaquePointer?>) -> Void {
    _exception.pointee = nil
    let sarg0 = CloudStatus(rawValue: arg0)
    var varg1 = arg1
    easyar_ListOfTarget_copy(varg1, &varg1)
    let sarg1 = ListOfTarget_from_c(OpaquePointerContainer(Optional.some(varg1)!, { p in easyar_ListOfTarget__dtor(p) }))
    let f = UnsafeMutablePointer<(CloudStatus, [Target]) -> Void>(state).pointee
    f(sarg0, sarg1)
}
fileprivate func FunctorOfVoidFromCloudStatusAndListOfTarget_to_c(_ f: @escaping (CloudStatus, [Target]) -> Void) -> easyar_FunctorOfVoidFromCloudStatusAndListOfTarget {
    let s = UnsafeMutablePointer<(CloudStatus, [Target]) -> Void>.allocate(capacity: 1)
    s.initialize(to: f)
    return easyar_FunctorOfVoidFromCloudStatusAndListOfTarget(_state: OpaquePointer(s), _func: FunctorOfVoidFromCloudStatusAndListOfTarget_func, _destroy: { p in UnsafeMutablePointer<(CloudStatus, [Target]) -> Void>(p).deinitialize(count: 1).deallocate() })
}

fileprivate func ListOfBlockInfo_to_c(_ l: [BlockInfo]) -> OpaquePointerContainer {
    let arr = l.map({ e in BlockInfo_to_c(e) })
    let pptr = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
    defer {
        pptr.deallocate()
    }
    arr.withUnsafeBufferPointer { bp in
        easyar_ListOfBlockInfo__ctor(UnsafePointer<easyar_BlockInfo>(bp.baseAddress), UnsafePointer<easyar_BlockInfo>(bp.baseAddress?.advanced(by: bp.count)), pptr)
    }
    return OpaquePointerContainer(pptr.pointee, { p in easyar_ListOfBlockInfo__dtor(p) })
}
fileprivate func ListOfBlockInfo_from_c(_ l: OpaquePointerContainer) -> [BlockInfo] {
    let size = Int(easyar_ListOfBlockInfo_size(l.ptr_))
    var values = [BlockInfo]()
    values.reserveCapacity(size)
    for k in 0..<size {
        let v = easyar_ListOfBlockInfo_at(l.ptr_, Int32(k))
        values.append(BlockInfo_from_c(v))
    }
    return values
}

fileprivate func FunctorOfVoidFromInputFrame_func(_ state: OpaquePointer, _ arg0: OpaquePointer, _ _exception: UnsafeMutablePointer<OpaquePointer?>) -> Void {
    _exception.pointee = nil
    var varg0 = arg0
    easyar_InputFrame__retain(varg0, &varg0)
    let sarg0 = (Object_from_c(Optional.some(varg0)!, easyar_InputFrame__typeName) as InputFrame)
    let f = UnsafeMutablePointer<(InputFrame) -> Void>(state).pointee
    f(sarg0)
}
fileprivate func FunctorOfVoidFromInputFrame_to_c(_ f: @escaping (InputFrame) -> Void) -> easyar_FunctorOfVoidFromInputFrame {
    let s = UnsafeMutablePointer<(InputFrame) -> Void>.allocate(capacity: 1)
    s.initialize(to: f)
    return easyar_FunctorOfVoidFromInputFrame(_state: OpaquePointer(s), _func: FunctorOfVoidFromInputFrame_func, _destroy: { p in UnsafeMutablePointer<(InputFrame) -> Void>(p).deinitialize(count: 1).deallocate() })
}

fileprivate func ListOfImageTarget_to_c(_ l: [ImageTarget]) -> OpaquePointerContainer {
    let arr = l.map({ e in e.cdata_ })
    let pptr = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
    defer {
        pptr.deallocate()
    }
    arr.withUnsafeBufferPointer { bp in
        easyar_ListOfImageTarget__ctor(UnsafePointer<OpaquePointer>(bp.baseAddress), UnsafePointer<OpaquePointer>(bp.baseAddress?.advanced(by: bp.count)), pptr)
    }
    return OpaquePointerContainer(pptr.pointee, { p in easyar_ListOfImageTarget__dtor(p) })
}
fileprivate func ListOfImageTarget_from_c(_ l: OpaquePointerContainer) -> [ImageTarget] {
    let size = Int(easyar_ListOfImageTarget_size(l.ptr_))
    var values = [ImageTarget]()
    values.reserveCapacity(size)
    for k in 0..<size {
        var v = easyar_ListOfImageTarget_at(l.ptr_, Int32(k))
        easyar_ImageTarget__retain(v, &v)
        values.append((Object_from_c(Optional.some(v)!, easyar_ImageTarget__typeName) as ImageTarget))
    }
    return values
}

fileprivate func ListOfImage_to_c(_ l: [Image]) -> OpaquePointerContainer {
    let arr = l.map({ e in e.cdata_ })
    let pptr = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
    defer {
        pptr.deallocate()
    }
    arr.withUnsafeBufferPointer { bp in
        easyar_ListOfImage__ctor(UnsafePointer<OpaquePointer>(bp.baseAddress), UnsafePointer<OpaquePointer>(bp.baseAddress?.advanced(by: bp.count)), pptr)
    }
    return OpaquePointerContainer(pptr.pointee, { p in easyar_ListOfImage__dtor(p) })
}
fileprivate func ListOfImage_from_c(_ l: OpaquePointerContainer) -> [Image] {
    let size = Int(easyar_ListOfImage_size(l.ptr_))
    var values = [Image]()
    values.reserveCapacity(size)
    for k in 0..<size {
        var v = easyar_ListOfImage_at(l.ptr_, Int32(k))
        easyar_Image__retain(v, &v)
        values.append((Object_from_c(Optional.some(v)!, easyar_Image__typeName) as Image))
    }
    return values
}

fileprivate func FunctorOfVoidFromPermissionStatusAndString_func(_ state: OpaquePointer, _ arg0: Int32, _ arg1: OpaquePointer, _ _exception: UnsafeMutablePointer<OpaquePointer?>) -> Void {
    _exception.pointee = nil
    let sarg0 = PermissionStatus(rawValue: arg0)
    var varg1 = arg1
    easyar_String_copy(varg1, &varg1)
    let sarg1 = String_from_c(OpaquePointerContainer(Optional.some(varg1)!, { p in easyar_String__dtor(p) }))
    let f = UnsafeMutablePointer<(PermissionStatus, String) -> Void>(state).pointee
    f(sarg0, sarg1)
}
fileprivate func FunctorOfVoidFromPermissionStatusAndString_to_c(_ f: @escaping (PermissionStatus, String) -> Void) -> easyar_FunctorOfVoidFromPermissionStatusAndString {
    let s = UnsafeMutablePointer<(PermissionStatus, String) -> Void>.allocate(capacity: 1)
    s.initialize(to: f)
    return easyar_FunctorOfVoidFromPermissionStatusAndString(_state: OpaquePointer(s), _func: FunctorOfVoidFromPermissionStatusAndString_func, _destroy: { p in UnsafeMutablePointer<(PermissionStatus, String) -> Void>(p).deinitialize(count: 1).deallocate() })
}

fileprivate func FunctorOfVoidFromRecordStatusAndString_func(_ state: OpaquePointer, _ arg0: Int32, _ arg1: OpaquePointer, _ _exception: UnsafeMutablePointer<OpaquePointer?>) -> Void {
    _exception.pointee = nil
    let sarg0 = RecordStatus(rawValue: arg0)
    var varg1 = arg1
    easyar_String_copy(varg1, &varg1)
    let sarg1 = String_from_c(OpaquePointerContainer(Optional.some(varg1)!, { p in easyar_String__dtor(p) }))
    let f = UnsafeMutablePointer<(RecordStatus, String) -> Void>(state).pointee
    f(sarg0, sarg1)
}
fileprivate func FunctorOfVoidFromRecordStatusAndString_to_c(_ f: @escaping (RecordStatus, String) -> Void) -> easyar_FunctorOfVoidFromRecordStatusAndString {
    let s = UnsafeMutablePointer<(RecordStatus, String) -> Void>.allocate(capacity: 1)
    s.initialize(to: f)
    return easyar_FunctorOfVoidFromRecordStatusAndString(_state: OpaquePointer(s), _func: FunctorOfVoidFromRecordStatusAndString_func, _destroy: { p in UnsafeMutablePointer<(RecordStatus, String) -> Void>(p).deinitialize(count: 1).deallocate() })
}

fileprivate func ListOfPlaneData_to_c(_ l: [PlaneData]) -> OpaquePointerContainer {
    let arr = l.map({ e in e.cdata_ })
    let pptr = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
    defer {
        pptr.deallocate()
    }
    arr.withUnsafeBufferPointer { bp in
        easyar_ListOfPlaneData__ctor(UnsafePointer<OpaquePointer>(bp.baseAddress), UnsafePointer<OpaquePointer>(bp.baseAddress?.advanced(by: bp.count)), pptr)
    }
    return OpaquePointerContainer(pptr.pointee, { p in easyar_ListOfPlaneData__dtor(p) })
}
fileprivate func ListOfPlaneData_from_c(_ l: OpaquePointerContainer) -> [PlaneData] {
    let size = Int(easyar_ListOfPlaneData_size(l.ptr_))
    var values = [PlaneData]()
    values.reserveCapacity(size)
    for k in 0..<size {
        var v = easyar_ListOfPlaneData_at(l.ptr_, Int32(k))
        easyar_PlaneData__retain(v, &v)
        values.append((Object_from_c(Optional.some(v)!, easyar_PlaneData__typeName) as PlaneData))
    }
    return values
}

fileprivate func FunctorOfVoidFromOptionalOfBuffer_func(_ state: OpaquePointer, _ arg0: easyar_OptionalOfBuffer, _ _exception: UnsafeMutablePointer<OpaquePointer?>) -> Void {
    _exception.pointee = nil
    var varg0 = arg0
    varg0.value = Optional_retain(varg0.value, easyar_Buffer__retain)
    let sarg0 = (varg0.has_value ? (Object_from_c(Optional.some(varg0.value!)!, easyar_Buffer__typeName) as Buffer) : nil)
    let f = UnsafeMutablePointer<(Buffer?) -> Void>(state).pointee
    f(sarg0)
}
fileprivate func FunctorOfVoidFromOptionalOfBuffer_to_c(_ f: @escaping (Buffer?) -> Void) -> easyar_FunctorOfVoidFromOptionalOfBuffer {
    let s = UnsafeMutablePointer<(Buffer?) -> Void>.allocate(capacity: 1)
    s.initialize(to: f)
    return easyar_FunctorOfVoidFromOptionalOfBuffer(_state: OpaquePointer(s), _func: FunctorOfVoidFromOptionalOfBuffer_func, _destroy: { p in UnsafeMutablePointer<(Buffer?) -> Void>(p).deinitialize(count: 1).deallocate() })
}

fileprivate func FunctorOfVoidFromBool_func(_ state: OpaquePointer, _ arg0: Bool, _ _exception: UnsafeMutablePointer<OpaquePointer?>) -> Void {
    _exception.pointee = nil
    let sarg0 = arg0
    let f = UnsafeMutablePointer<(Bool) -> Void>(state).pointee
    f(sarg0)
}
fileprivate func FunctorOfVoidFromBool_to_c(_ f: @escaping (Bool) -> Void) -> easyar_FunctorOfVoidFromBool {
    let s = UnsafeMutablePointer<(Bool) -> Void>.allocate(capacity: 1)
    s.initialize(to: f)
    return easyar_FunctorOfVoidFromBool(_state: OpaquePointer(s), _func: FunctorOfVoidFromBool_func, _destroy: { p in UnsafeMutablePointer<(Bool) -> Void>(p).deinitialize(count: 1).deallocate() })
}

fileprivate func FunctorOfVoidFromCameraState_func(_ state: OpaquePointer, _ arg0: Int32, _ _exception: UnsafeMutablePointer<OpaquePointer?>) -> Void {
    _exception.pointee = nil
    let sarg0 = CameraState(rawValue: arg0)
    let f = UnsafeMutablePointer<(CameraState) -> Void>(state).pointee
    f(sarg0)
}
fileprivate func FunctorOfVoidFromCameraState_to_c(_ f: @escaping (CameraState) -> Void) -> easyar_FunctorOfVoidFromCameraState {
    let s = UnsafeMutablePointer<(CameraState) -> Void>.allocate(capacity: 1)
    s.initialize(to: f)
    return easyar_FunctorOfVoidFromCameraState(_state: OpaquePointer(s), _func: FunctorOfVoidFromCameraState_func, _destroy: { p in UnsafeMutablePointer<(CameraState) -> Void>(p).deinitialize(count: 1).deallocate() })
}

fileprivate func FunctorOfVoidFromFeedbackFrame_func(_ state: OpaquePointer, _ arg0: OpaquePointer, _ _exception: UnsafeMutablePointer<OpaquePointer?>) -> Void {
    _exception.pointee = nil
    var varg0 = arg0
    easyar_FeedbackFrame__retain(varg0, &varg0)
    let sarg0 = (Object_from_c(Optional.some(varg0)!, easyar_FeedbackFrame__typeName) as FeedbackFrame)
    let f = UnsafeMutablePointer<(FeedbackFrame) -> Void>(state).pointee
    f(sarg0)
}
fileprivate func FunctorOfVoidFromFeedbackFrame_to_c(_ f: @escaping (FeedbackFrame) -> Void) -> easyar_FunctorOfVoidFromFeedbackFrame {
    let s = UnsafeMutablePointer<(FeedbackFrame) -> Void>.allocate(capacity: 1)
    s.initialize(to: f)
    return easyar_FunctorOfVoidFromFeedbackFrame(_state: OpaquePointer(s), _func: FunctorOfVoidFromFeedbackFrame_func, _destroy: { p in UnsafeMutablePointer<(FeedbackFrame) -> Void>(p).deinitialize(count: 1).deallocate() })
}

fileprivate func FunctorOfOutputFrameFromListOfOutputFrame_func(_ state: OpaquePointer, _ arg0: OpaquePointer, _ Return: UnsafeMutablePointer<OpaquePointer>, _ _exception: UnsafeMutablePointer<OpaquePointer?>) -> Void {
    _exception.pointee = nil
    var varg0 = arg0
    easyar_ListOfOutputFrame_copy(varg0, &varg0)
    let sarg0 = ListOfOutputFrame_from_c(OpaquePointerContainer(Optional.some(varg0)!, { p in easyar_ListOfOutputFrame__dtor(p) }))
    let f = UnsafeMutablePointer<([OutputFrame]) -> OutputFrame>(state).pointee
    let _return_value_ = f(sarg0)
    var _return_value_c_ = _return_value_.cdata_
    easyar_OutputFrame__retain(_return_value_c_, &_return_value_c_)
    Return.pointee = _return_value_c_
}
fileprivate func FunctorOfOutputFrameFromListOfOutputFrame_to_c(_ f: @escaping ([OutputFrame]) -> OutputFrame) -> easyar_FunctorOfOutputFrameFromListOfOutputFrame {
    let s = UnsafeMutablePointer<([OutputFrame]) -> OutputFrame>.allocate(capacity: 1)
    s.initialize(to: f)
    return easyar_FunctorOfOutputFrameFromListOfOutputFrame(_state: OpaquePointer(s), _func: FunctorOfOutputFrameFromListOfOutputFrame_func, _destroy: { p in UnsafeMutablePointer<([OutputFrame]) -> OutputFrame>(p).deinitialize(count: 1).deallocate() })
}

fileprivate func ListOfOutputFrame_to_c(_ l: [OutputFrame]) -> OpaquePointerContainer {
    let arr = l.map({ e in e.cdata_ })
    let pptr = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
    defer {
        pptr.deallocate()
    }
    arr.withUnsafeBufferPointer { bp in
        easyar_ListOfOutputFrame__ctor(UnsafePointer<OpaquePointer>(bp.baseAddress), UnsafePointer<OpaquePointer>(bp.baseAddress?.advanced(by: bp.count)), pptr)
    }
    return OpaquePointerContainer(pptr.pointee, { p in easyar_ListOfOutputFrame__dtor(p) })
}
fileprivate func ListOfOutputFrame_from_c(_ l: OpaquePointerContainer) -> [OutputFrame] {
    let size = Int(easyar_ListOfOutputFrame_size(l.ptr_))
    var values = [OutputFrame]()
    values.reserveCapacity(size)
    for k in 0..<size {
        var v = easyar_ListOfOutputFrame_at(l.ptr_, Int32(k))
        easyar_OutputFrame__retain(v, &v)
        values.append((Object_from_c(Optional.some(v)!, easyar_OutputFrame__typeName) as OutputFrame))
    }
    return values
}

fileprivate func FunctorOfVoidFromVideoStatus_func(_ state: OpaquePointer, _ arg0: Int32, _ _exception: UnsafeMutablePointer<OpaquePointer?>) -> Void {
    _exception.pointee = nil
    let sarg0 = VideoStatus(rawValue: arg0)
    let f = UnsafeMutablePointer<(VideoStatus) -> Void>(state).pointee
    f(sarg0)
}
fileprivate func FunctorOfVoidFromVideoStatus_to_c(_ f: @escaping (VideoStatus) -> Void) -> easyar_FunctorOfVoidFromVideoStatus {
    let s = UnsafeMutablePointer<(VideoStatus) -> Void>.allocate(capacity: 1)
    s.initialize(to: f)
    return easyar_FunctorOfVoidFromVideoStatus(_state: OpaquePointer(s), _func: FunctorOfVoidFromVideoStatus_func, _destroy: { p in UnsafeMutablePointer<(VideoStatus) -> Void>(p).deinitialize(count: 1).deallocate() })
}

fileprivate func FunctorOfVoidFromLogLevelAndString_func(_ state: OpaquePointer, _ arg0: Int32, _ arg1: OpaquePointer, _ _exception: UnsafeMutablePointer<OpaquePointer?>) -> Void {
    _exception.pointee = nil
    let sarg0 = LogLevel(rawValue: arg0)
    var varg1 = arg1
    easyar_String_copy(varg1, &varg1)
    let sarg1 = String_from_c(OpaquePointerContainer(Optional.some(varg1)!, { p in easyar_String__dtor(p) }))
    let f = UnsafeMutablePointer<(LogLevel, String) -> Void>(state).pointee
    f(sarg0, sarg1)
}
fileprivate func FunctorOfVoidFromLogLevelAndString_to_c(_ f: @escaping (LogLevel, String) -> Void) -> easyar_FunctorOfVoidFromLogLevelAndString {
    let s = UnsafeMutablePointer<(LogLevel, String) -> Void>.allocate(capacity: 1)
    s.initialize(to: f)
    return easyar_FunctorOfVoidFromLogLevelAndString(_state: OpaquePointer(s), _func: FunctorOfVoidFromLogLevelAndString_func, _destroy: { p in UnsafeMutablePointer<(LogLevel, String) -> Void>(p).deinitialize(count: 1).deallocate() })
}

public class ObjectTargetParameters : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public convenience init() {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ObjectTargetParameters__ctor(_return_value_)
        self.init(cdata: _return_value_.pointee, deleter: { ptr in easyar_ObjectTargetParameters__dtor(ptr) })
    }
    public func bufferDictionary() -> BufferDictionary {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ObjectTargetParameters_bufferDictionary(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_BufferDictionary__typeName) as BufferDictionary)
    }
    public func setBufferDictionary(_ bufferDictionary: BufferDictionary) -> Void {
        easyar_ObjectTargetParameters_setBufferDictionary(cdata_, bufferDictionary.cdata_)
    }
    public func objPath() -> String {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ObjectTargetParameters_objPath(cdata_, _return_value_)
        return String_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_String__dtor(p) }))
    }
    public func setObjPath(_ objPath: String) -> Void {
        easyar_ObjectTargetParameters_setObjPath(cdata_, String_to_c(objPath).ptr_)
    }
    public func name() -> String {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ObjectTargetParameters_name(cdata_, _return_value_)
        return String_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_String__dtor(p) }))
    }
    public func setName(_ name: String) -> Void {
        easyar_ObjectTargetParameters_setName(cdata_, String_to_c(name).ptr_)
    }
    public func uid() -> String {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ObjectTargetParameters_uid(cdata_, _return_value_)
        return String_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_String__dtor(p) }))
    }
    public func setUid(_ uid: String) -> Void {
        easyar_ObjectTargetParameters_setUid(cdata_, String_to_c(uid).ptr_)
    }
    public func meta() -> String {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ObjectTargetParameters_meta(cdata_, _return_value_)
        return String_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_String__dtor(p) }))
    }
    public func setMeta(_ meta: String) -> Void {
        easyar_ObjectTargetParameters_setMeta(cdata_, String_to_c(meta).ptr_)
    }
    public func scale() -> Float {
        let _return_value_ = easyar_ObjectTargetParameters_scale(cdata_)
        return _return_value_
    }
    public func setScale(_ size: Float) -> Void {
        easyar_ObjectTargetParameters_setScale(cdata_, size)
    }
}

public class ObjectTarget : Target {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public convenience init() {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ObjectTarget__ctor(_return_value_)
        self.init(cdata: _return_value_.pointee, deleter: { ptr in easyar_ObjectTarget__dtor(ptr) })
    }
    public static func createFromParameters(_ parameters: ObjectTargetParameters) -> ObjectTarget? {
        let _return_value_ = UnsafeMutablePointer<easyar_OptionalOfObjectTarget>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ObjectTarget_createFromParameters(parameters.cdata_, _return_value_)
        return (_return_value_.pointee.has_value ? (Object_from_c(Optional.some(_return_value_.pointee.value!)!, easyar_ObjectTarget__typeName) as ObjectTarget) : nil)
    }
    public static func createFromObjectFile(_ path: String, _ storageType: StorageType, _ name: String, _ uid: String, _ meta: String, _ scale: Float) -> ObjectTarget? {
        let _return_value_ = UnsafeMutablePointer<easyar_OptionalOfObjectTarget>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ObjectTarget_createFromObjectFile(String_to_c(path).ptr_, storageType.rawValue, String_to_c(name).ptr_, String_to_c(uid).ptr_, String_to_c(meta).ptr_, scale, _return_value_)
        return (_return_value_.pointee.has_value ? (Object_from_c(Optional.some(_return_value_.pointee.value!)!, easyar_ObjectTarget__typeName) as ObjectTarget) : nil)
    }
    public static func setupAll(_ path: String, _ storageType: StorageType) -> [ObjectTarget] {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ObjectTarget_setupAll(String_to_c(path).ptr_, storageType.rawValue, _return_value_)
        return ListOfObjectTarget_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_ListOfObjectTarget__dtor(p) }))
    }
    public func scale() -> Float {
        let _return_value_ = easyar_ObjectTarget_scale(cdata_)
        return _return_value_
    }
    public func boundingBox() -> [Vec3F] {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ObjectTarget_boundingBox(cdata_, _return_value_)
        return ListOfVec3F_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_ListOfVec3F__dtor(p) }))
    }
    public func setScale(_ scale: Float) -> Bool {
        let _return_value_ = easyar_ObjectTarget_setScale(cdata_, scale)
        return _return_value_
    }
    public override func runtimeID() -> Int32 {
        let _return_value_ = easyar_ObjectTarget_runtimeID(cdata_)
        return _return_value_
    }
    public override func uid() -> String {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ObjectTarget_uid(cdata_, _return_value_)
        return String_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_String__dtor(p) }))
    }
    public override func name() -> String {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ObjectTarget_name(cdata_, _return_value_)
        return String_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_String__dtor(p) }))
    }
    public override func setName(_ name: String) -> Void {
        easyar_ObjectTarget_setName(cdata_, String_to_c(name).ptr_)
    }
    public override func meta() -> String {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ObjectTarget_meta(cdata_, _return_value_)
        return String_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_String__dtor(p) }))
    }
    public override func setMeta(_ data: String) -> Void {
        easyar_ObjectTarget_setMeta(cdata_, String_to_c(data).ptr_)
    }
}

public class ObjectTrackerResult : TargetTrackerResult {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public override func targetInstances() -> [TargetInstance] {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ObjectTrackerResult_targetInstances(cdata_, _return_value_)
        return ListOfTargetInstance_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_ListOfTargetInstance__dtor(p) }))
    }
    public override func setTargetInstances(_ instances: [TargetInstance]) -> Void {
        easyar_ObjectTrackerResult_setTargetInstances(cdata_, ListOfTargetInstance_to_c(instances).ptr_)
    }
}

public class ObjectTracker : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public static func isAvailable() -> Bool {
        let _return_value_ = easyar_ObjectTracker_isAvailable()
        return _return_value_
    }
    public func feedbackFrameSink() -> FeedbackFrameSink {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ObjectTracker_feedbackFrameSink(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_FeedbackFrameSink__typeName) as FeedbackFrameSink)
    }
    public func outputFrameSource() -> OutputFrameSource {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ObjectTracker_outputFrameSource(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_OutputFrameSource__typeName) as OutputFrameSource)
    }
    public static func create() -> ObjectTracker {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ObjectTracker_create(_return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_ObjectTracker__typeName) as ObjectTracker)
    }
    public func start() -> Bool {
        let _return_value_ = easyar_ObjectTracker_start(cdata_)
        return _return_value_
    }
    public func stop() -> Void {
        easyar_ObjectTracker_stop(cdata_)
    }
    public func close() -> Void {
        easyar_ObjectTracker_close(cdata_)
    }
    public func loadTarget(_ target: Target, _ callbackScheduler: CallbackScheduler, _ callback: @escaping (Target, Bool) -> Void) -> Void {
        easyar_ObjectTracker_loadTarget(cdata_, target.cdata_, callbackScheduler.cdata_, FunctorOfVoidFromTargetAndBool_to_c(callback))
    }
    public func unloadTarget(_ target: Target, _ callbackScheduler: CallbackScheduler, _ callback: @escaping (Target, Bool) -> Void) -> Void {
        easyar_ObjectTracker_unloadTarget(cdata_, target.cdata_, callbackScheduler.cdata_, FunctorOfVoidFromTargetAndBool_to_c(callback))
    }
    public func targets() -> [Target] {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ObjectTracker_targets(cdata_, _return_value_)
        return ListOfTarget_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_ListOfTarget__dtor(p) }))
    }
    public func setSimultaneousNum(_ num: Int32) -> Bool {
        let _return_value_ = easyar_ObjectTracker_setSimultaneousNum(cdata_, num)
        return _return_value_
    }
    public func simultaneousNum() -> Int32 {
        let _return_value_ = easyar_ObjectTracker_simultaneousNum(cdata_)
        return _return_value_
    }
}

public class CADTarget : Target {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public convenience init() {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_CADTarget__ctor(_return_value_)
        self.init(cdata: _return_value_.pointee, deleter: { ptr in easyar_CADTarget__dtor(ptr) })
    }
    public static func createFromUnity(_ vertices: Buffer, _ faces: Buffer, _ transform: Matrix44F, _ localScale: Float) -> CADTarget? {
        let _return_value_ = UnsafeMutablePointer<easyar_OptionalOfCADTarget>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_CADTarget_createFromUnity(vertices.cdata_, faces.cdata_, Matrix44F_to_c(transform), localScale, _return_value_)
        return (_return_value_.pointee.has_value ? (Object_from_c(Optional.some(_return_value_.pointee.value!)!, easyar_CADTarget__typeName) as CADTarget) : nil)
    }
    public func updateTransform(_ transform: Matrix44F, _ localScale: Float) -> Void {
        easyar_CADTarget_updateTransform(cdata_, Matrix44F_to_c(transform), localScale)
    }
    public func updateTrackConfig(_ threshContrast: Float, _ threshConsist: Float, _ threshQuality: Float) -> Void {
        easyar_CADTarget_updateTrackConfig(cdata_, threshContrast, threshConsist, threshQuality)
    }
    public static func createFromObjectFile(_ path: String, _ storageType: StorageType, _ name: String, _ uid: String, _ meta: String, _ scale: Float) -> CADTarget? {
        let _return_value_ = UnsafeMutablePointer<easyar_OptionalOfCADTarget>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_CADTarget_createFromObjectFile(String_to_c(path).ptr_, storageType.rawValue, String_to_c(name).ptr_, String_to_c(uid).ptr_, String_to_c(meta).ptr_, scale, _return_value_)
        return (_return_value_.pointee.has_value ? (Object_from_c(Optional.some(_return_value_.pointee.value!)!, easyar_CADTarget__typeName) as CADTarget) : nil)
    }
    public static func setupAll(_ path: String, _ storageType: StorageType) -> [CADTarget] {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_CADTarget_setupAll(String_to_c(path).ptr_, storageType.rawValue, _return_value_)
        return ListOfCADTarget_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_ListOfCADTarget__dtor(p) }))
    }
    public func scale() -> Float {
        let _return_value_ = easyar_CADTarget_scale(cdata_)
        return _return_value_
    }
    public func boundingBox() -> [Vec3F] {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_CADTarget_boundingBox(cdata_, _return_value_)
        return ListOfVec3F_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_ListOfVec3F__dtor(p) }))
    }
    public func boundingBoxGL() -> [Vec3F] {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_CADTarget_boundingBoxGL(cdata_, _return_value_)
        return ListOfVec3F_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_ListOfVec3F__dtor(p) }))
    }
    public func setScale(_ scale: Float) -> Bool {
        let _return_value_ = easyar_CADTarget_setScale(cdata_, scale)
        return _return_value_
    }
    public override func runtimeID() -> Int32 {
        let _return_value_ = easyar_CADTarget_runtimeID(cdata_)
        return _return_value_
    }
    public override func uid() -> String {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_CADTarget_uid(cdata_, _return_value_)
        return String_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_String__dtor(p) }))
    }
    public override func name() -> String {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_CADTarget_name(cdata_, _return_value_)
        return String_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_String__dtor(p) }))
    }
    public override func setName(_ name: String) -> Void {
        easyar_CADTarget_setName(cdata_, String_to_c(name).ptr_)
    }
    public override func meta() -> String {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_CADTarget_meta(cdata_, _return_value_)
        return String_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_String__dtor(p) }))
    }
    public override func setMeta(_ data: String) -> Void {
        easyar_CADTarget_setMeta(cdata_, String_to_c(data).ptr_)
    }
}

public class CADTrackerResult : TargetTrackerResult {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public override func targetInstances() -> [TargetInstance] {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_CADTrackerResult_targetInstances(cdata_, _return_value_)
        return ListOfTargetInstance_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_ListOfTargetInstance__dtor(p) }))
    }
    public override func setTargetInstances(_ instances: [TargetInstance]) -> Void {
        easyar_CADTrackerResult_setTargetInstances(cdata_, ListOfTargetInstance_to_c(instances).ptr_)
    }
}

public class CADTracker : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public static func isAvailable() -> Bool {
        let _return_value_ = easyar_CADTracker_isAvailable()
        return _return_value_
    }
    public func feedbackFrameSink() -> FeedbackFrameSink {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_CADTracker_feedbackFrameSink(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_FeedbackFrameSink__typeName) as FeedbackFrameSink)
    }
    public func outputFrameSource() -> OutputFrameSource {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_CADTracker_outputFrameSource(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_OutputFrameSource__typeName) as OutputFrameSource)
    }
    public static func create() -> CADTracker {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_CADTracker_create(_return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_CADTracker__typeName) as CADTracker)
    }
    public func start() -> Bool {
        let _return_value_ = easyar_CADTracker_start(cdata_)
        return _return_value_
    }
    public func stop() -> Void {
        easyar_CADTracker_stop(cdata_)
    }
    public func close() -> Void {
        easyar_CADTracker_close(cdata_)
    }
    public func loadTarget(_ target: Target, _ callbackScheduler: CallbackScheduler, _ callback: @escaping (Target, Bool) -> Void) -> Void {
        easyar_CADTracker_loadTarget(cdata_, target.cdata_, callbackScheduler.cdata_, FunctorOfVoidFromTargetAndBool_to_c(callback))
    }
    public func unloadTarget(_ target: Target, _ callbackScheduler: CallbackScheduler, _ callback: @escaping (Target, Bool) -> Void) -> Void {
        easyar_CADTracker_unloadTarget(cdata_, target.cdata_, callbackScheduler.cdata_, FunctorOfVoidFromTargetAndBool_to_c(callback))
    }
    public func targets() -> [Target] {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_CADTracker_targets(cdata_, _return_value_)
        return ListOfTarget_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_ListOfTarget__dtor(p) }))
    }
}

public struct CloudStatus : OptionSet {
    public let rawValue: Int32
    public init(rawValue: Int32) { self.rawValue = rawValue }

    public static let FoundTargets = CloudStatus(rawValue: 0)
    public static let TargetsNotFound = CloudStatus(rawValue: 1)
    public static let Reconnecting = CloudStatus(rawValue: 2)
    public static let ProtocolError = CloudStatus(rawValue: 3)
}

public class CloudRecognizer : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public static func isAvailable() -> Bool {
        let _return_value_ = easyar_CloudRecognizer_isAvailable()
        return _return_value_
    }
    public func inputFrameSink() -> InputFrameSink {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_CloudRecognizer_inputFrameSink(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_InputFrameSink__typeName) as InputFrameSink)
    }
    public static func create(_ server: String, _ appKey: String, _ appSecret: String, _ callbackScheduler: CallbackScheduler, _ callback: ((CloudStatus, [Target]) -> Void)?) -> CloudRecognizer {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_CloudRecognizer_create(String_to_c(server).ptr_, String_to_c(appKey).ptr_, String_to_c(appSecret).ptr_, callbackScheduler.cdata_, (callback.map({(p) in easyar_OptionalOfFunctorOfVoidFromCloudStatusAndListOfTarget(has_value: true, value: FunctorOfVoidFromCloudStatusAndListOfTarget_to_c(p))}) ?? easyar_OptionalOfFunctorOfVoidFromCloudStatusAndListOfTarget(has_value: false, value: defaultValue())), _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_CloudRecognizer__typeName) as CloudRecognizer)
    }
    public func start() -> Bool {
        let _return_value_ = easyar_CloudRecognizer_start(cdata_)
        return _return_value_
    }
    public func stop() -> Void {
        easyar_CloudRecognizer_stop(cdata_)
    }
    public func close() -> Void {
        easyar_CloudRecognizer_close(cdata_)
    }
}

public class DenseSpatialMap : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public static func isAvailable() -> Bool {
        let _return_value_ = easyar_DenseSpatialMap_isAvailable()
        return _return_value_
    }
    public func inputFrameSink() -> InputFrameSink {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_DenseSpatialMap_inputFrameSink(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_InputFrameSink__typeName) as InputFrameSink)
    }
    public static func create() -> DenseSpatialMap {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_DenseSpatialMap_create(_return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_DenseSpatialMap__typeName) as DenseSpatialMap)
    }
    public func start() -> Bool {
        let _return_value_ = easyar_DenseSpatialMap_start(cdata_)
        return _return_value_
    }
    public func stop() -> Void {
        easyar_DenseSpatialMap_stop(cdata_)
    }
    public func close() -> Void {
        easyar_DenseSpatialMap_close(cdata_)
    }
    public func saveMap() -> String {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_DenseSpatialMap_saveMap(cdata_, _return_value_)
        return String_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_String__dtor(p) }))
    }
    public func getMesh() -> SceneMesh {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_DenseSpatialMap_getMesh(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_SceneMesh__typeName) as SceneMesh)
    }
    public func updateSceneMesh(_ updateMeshAll: Bool) -> Bool {
        let _return_value_ = easyar_DenseSpatialMap_updateSceneMesh(cdata_, updateMeshAll)
        return _return_value_
    }
}

public struct BlockInfo {
    public var x_: Int32
    public var y_: Int32
    public var z_: Int32
    public var numOfVertex_: Int32
    public var startPointOfVertex_: Int32
    public var numOfIndex_: Int32
    public var startPointOfIndex_: Int32
    public var age_: Int32

    public init(_ x_: Int32, _ y_: Int32, _ z_: Int32, _ numOfVertex_: Int32, _ startPointOfVertex_: Int32, _ numOfIndex_: Int32, _ startPointOfIndex_: Int32, _ age_: Int32) {
        self.x_ = x_;
        self.y_ = y_;
        self.z_ = z_;
        self.numOfVertex_ = numOfVertex_;
        self.startPointOfVertex_ = startPointOfVertex_;
        self.numOfIndex_ = numOfIndex_;
        self.startPointOfIndex_ = startPointOfIndex_;
        self.age_ = age_;
    }
}

public class SceneMesh : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public convenience init() {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_SceneMesh__ctor(_return_value_)
        self.init(cdata: _return_value_.pointee, deleter: { ptr in easyar_SceneMesh__dtor(ptr) })
    }
    public func getNumOfVertexAll() -> Int32 {
        let _return_value_ = easyar_SceneMesh_getNumOfVertexAll(cdata_)
        return _return_value_
    }
    public func getNumOfIndexAll() -> Int32 {
        let _return_value_ = easyar_SceneMesh_getNumOfIndexAll(cdata_)
        return _return_value_
    }
    public func getVerticesAll() -> Buffer {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_SceneMesh_getVerticesAll(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_Buffer__typeName) as Buffer)
    }
    public func getNormalsAll() -> Buffer {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_SceneMesh_getNormalsAll(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_Buffer__typeName) as Buffer)
    }
    public func getIndicesAll() -> Buffer {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_SceneMesh_getIndicesAll(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_Buffer__typeName) as Buffer)
    }
    public func getNumOfVertexIncremental() -> Int32 {
        let _return_value_ = easyar_SceneMesh_getNumOfVertexIncremental(cdata_)
        return _return_value_
    }
    public func getNumOfIndexIncremental() -> Int32 {
        let _return_value_ = easyar_SceneMesh_getNumOfIndexIncremental(cdata_)
        return _return_value_
    }
    public func getVerticesIncremental() -> Buffer {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_SceneMesh_getVerticesIncremental(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_Buffer__typeName) as Buffer)
    }
    public func getNormalsIncremental() -> Buffer {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_SceneMesh_getNormalsIncremental(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_Buffer__typeName) as Buffer)
    }
    public func getIndicesIncremental() -> Buffer {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_SceneMesh_getIndicesIncremental(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_Buffer__typeName) as Buffer)
    }
    public func getBlocksInfoIncremental() -> [BlockInfo] {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_SceneMesh_getBlocksInfoIncremental(cdata_, _return_value_)
        return ListOfBlockInfo_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_ListOfBlockInfo__dtor(p) }))
    }
    public func getBlockDimentionInMeters() -> Float {
        let _return_value_ = easyar_SceneMesh_getBlockDimentionInMeters(cdata_)
        return _return_value_
    }
}

public struct SurfaceTrackingStatus : OptionSet {
    public let rawValue: Int32
    public init(rawValue: Int32) { self.rawValue = rawValue }

    public static let NotTracking = SurfaceTrackingStatus(rawValue: 0)
    public static let BadTracking = SurfaceTrackingStatus(rawValue: 1)
    public static let GoodTracking = SurfaceTrackingStatus(rawValue: 2)
}

public class SurfaceTrackerResult : FrameFilterResult {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public func transform() -> Matrix44F {
        let _return_value_ = easyar_SurfaceTrackerResult_transform(cdata_)
        return Matrix44F_from_c(_return_value_)
    }
    public func status() -> SurfaceTrackingStatus {
        let _return_value_ = easyar_SurfaceTrackerResult_status(cdata_)
        return SurfaceTrackingStatus(rawValue: _return_value_)
    }
}

public class SurfaceTracker : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public static func isAvailable() -> Bool {
        let _return_value_ = easyar_SurfaceTracker_isAvailable()
        return _return_value_
    }
    public func inputFrameSink() -> InputFrameSink {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_SurfaceTracker_inputFrameSink(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_InputFrameSink__typeName) as InputFrameSink)
    }
    public func outputFrameSource() -> OutputFrameSource {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_SurfaceTracker_outputFrameSource(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_OutputFrameSource__typeName) as OutputFrameSource)
    }
    public static func create() -> SurfaceTracker {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_SurfaceTracker_create(_return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_SurfaceTracker__typeName) as SurfaceTracker)
    }
    public func start() -> Bool {
        let _return_value_ = easyar_SurfaceTracker_start(cdata_)
        return _return_value_
    }
    public func stop() -> Void {
        easyar_SurfaceTracker_stop(cdata_)
    }
    public func close() -> Void {
        easyar_SurfaceTracker_close(cdata_)
    }
    public func alignTargetToCameraImagePoint(_ cameraImagePoint: Vec2F) -> Void {
        easyar_SurfaceTracker_alignTargetToCameraImagePoint(cdata_, Vec2F_to_c(cameraImagePoint))
    }
}

public struct MotionTrackingStatus : OptionSet {
    public let rawValue: Int32
    public init(rawValue: Int32) { self.rawValue = rawValue }

    public static let NotTracking = MotionTrackingStatus(rawValue: 0)
    public static let BadTracking = MotionTrackingStatus(rawValue: 1)
    public static let GoodTracking = MotionTrackingStatus(rawValue: 2)
}

public class MotionTrackerResult : FrameFilterResult {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public func transform() -> Matrix44F {
        let _return_value_ = easyar_MotionTrackerResult_transform(cdata_)
        return Matrix44F_from_c(_return_value_)
    }
    public func status() -> MotionTrackingStatus {
        let _return_value_ = easyar_MotionTrackerResult_status(cdata_)
        return MotionTrackingStatus(rawValue: _return_value_)
    }
}

public class MotionTracker : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public static func isAvailable() -> Bool {
        let _return_value_ = easyar_MotionTracker_isAvailable()
        return _return_value_
    }
    public func inputFrameSink() -> InputFrameSink {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_MotionTracker_inputFrameSink(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_InputFrameSink__typeName) as InputFrameSink)
    }
    public func outputFrameSource() -> OutputFrameSource {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_MotionTracker_outputFrameSource(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_OutputFrameSource__typeName) as OutputFrameSource)
    }
    public static func create() -> MotionTracker {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_MotionTracker_create(_return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_MotionTracker__typeName) as MotionTracker)
    }
    public func start() -> Bool {
        let _return_value_ = easyar_MotionTracker_start(cdata_)
        return _return_value_
    }
    public func stop() -> Void {
        easyar_MotionTracker_stop(cdata_)
    }
    public func close() -> Void {
        easyar_MotionTracker_close(cdata_)
    }
}

public class MotionTrackerOutputFrameToInputFrameAdapter : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public func input() -> OutputFrameSink {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_MotionTrackerOutputFrameToInputFrameAdapter_input(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_OutputFrameSink__typeName) as OutputFrameSink)
    }
    public func output() -> InputFrameSource {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_MotionTrackerOutputFrameToInputFrameAdapter_output(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_InputFrameSource__typeName) as InputFrameSource)
    }
    public static func create() -> MotionTrackerOutputFrameToInputFrameAdapter {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_MotionTrackerOutputFrameToInputFrameAdapter_create(_return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_MotionTrackerOutputFrameToInputFrameAdapter__typeName) as MotionTrackerOutputFrameToInputFrameAdapter)
    }
}

public class ImageTargetParameters : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public convenience init() {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ImageTargetParameters__ctor(_return_value_)
        self.init(cdata: _return_value_.pointee, deleter: { ptr in easyar_ImageTargetParameters__dtor(ptr) })
    }
    public func image() -> Image {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ImageTargetParameters_image(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_Image__typeName) as Image)
    }
    public func setImage(_ image: Image) -> Void {
        easyar_ImageTargetParameters_setImage(cdata_, image.cdata_)
    }
    public func name() -> String {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ImageTargetParameters_name(cdata_, _return_value_)
        return String_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_String__dtor(p) }))
    }
    public func setName(_ name: String) -> Void {
        easyar_ImageTargetParameters_setName(cdata_, String_to_c(name).ptr_)
    }
    public func uid() -> String {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ImageTargetParameters_uid(cdata_, _return_value_)
        return String_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_String__dtor(p) }))
    }
    public func setUid(_ uid: String) -> Void {
        easyar_ImageTargetParameters_setUid(cdata_, String_to_c(uid).ptr_)
    }
    public func meta() -> String {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ImageTargetParameters_meta(cdata_, _return_value_)
        return String_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_String__dtor(p) }))
    }
    public func setMeta(_ meta: String) -> Void {
        easyar_ImageTargetParameters_setMeta(cdata_, String_to_c(meta).ptr_)
    }
    public func scale() -> Float {
        let _return_value_ = easyar_ImageTargetParameters_scale(cdata_)
        return _return_value_
    }
    public func setScale(_ scale: Float) -> Void {
        easyar_ImageTargetParameters_setScale(cdata_, scale)
    }
}

public class ImageTarget : Target {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public convenience init() {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ImageTarget__ctor(_return_value_)
        self.init(cdata: _return_value_.pointee, deleter: { ptr in easyar_ImageTarget__dtor(ptr) })
    }
    public static func createFromParameters(_ parameters: ImageTargetParameters) -> ImageTarget? {
        let _return_value_ = UnsafeMutablePointer<easyar_OptionalOfImageTarget>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ImageTarget_createFromParameters(parameters.cdata_, _return_value_)
        return (_return_value_.pointee.has_value ? (Object_from_c(Optional.some(_return_value_.pointee.value!)!, easyar_ImageTarget__typeName) as ImageTarget) : nil)
    }
    public static func createFromTargetFile(_ path: String, _ storageType: StorageType) -> ImageTarget? {
        let _return_value_ = UnsafeMutablePointer<easyar_OptionalOfImageTarget>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ImageTarget_createFromTargetFile(String_to_c(path).ptr_, storageType.rawValue, _return_value_)
        return (_return_value_.pointee.has_value ? (Object_from_c(Optional.some(_return_value_.pointee.value!)!, easyar_ImageTarget__typeName) as ImageTarget) : nil)
    }
    public static func createFromTargetData(_ buffer: Buffer) -> ImageTarget? {
        let _return_value_ = UnsafeMutablePointer<easyar_OptionalOfImageTarget>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ImageTarget_createFromTargetData(buffer.cdata_, _return_value_)
        return (_return_value_.pointee.has_value ? (Object_from_c(Optional.some(_return_value_.pointee.value!)!, easyar_ImageTarget__typeName) as ImageTarget) : nil)
    }
    public func save(_ path: String) -> Bool {
        let _return_value_ = easyar_ImageTarget_save(cdata_, String_to_c(path).ptr_)
        return _return_value_
    }
    public static func createFromImageFile(_ path: String, _ storageType: StorageType, _ name: String, _ uid: String, _ meta: String, _ scale: Float) -> ImageTarget? {
        let _return_value_ = UnsafeMutablePointer<easyar_OptionalOfImageTarget>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ImageTarget_createFromImageFile(String_to_c(path).ptr_, storageType.rawValue, String_to_c(name).ptr_, String_to_c(uid).ptr_, String_to_c(meta).ptr_, scale, _return_value_)
        return (_return_value_.pointee.has_value ? (Object_from_c(Optional.some(_return_value_.pointee.value!)!, easyar_ImageTarget__typeName) as ImageTarget) : nil)
    }
    public static func setupAll(_ path: String, _ storageType: StorageType) -> [ImageTarget] {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ImageTarget_setupAll(String_to_c(path).ptr_, storageType.rawValue, _return_value_)
        return ListOfImageTarget_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_ListOfImageTarget__dtor(p) }))
    }
    public func scale() -> Float {
        let _return_value_ = easyar_ImageTarget_scale(cdata_)
        return _return_value_
    }
    public func aspectRatio() -> Float {
        let _return_value_ = easyar_ImageTarget_aspectRatio(cdata_)
        return _return_value_
    }
    public func setScale(_ scale: Float) -> Bool {
        let _return_value_ = easyar_ImageTarget_setScale(cdata_, scale)
        return _return_value_
    }
    public func images() -> [Image] {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ImageTarget_images(cdata_, _return_value_)
        return ListOfImage_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_ListOfImage__dtor(p) }))
    }
    public override func runtimeID() -> Int32 {
        let _return_value_ = easyar_ImageTarget_runtimeID(cdata_)
        return _return_value_
    }
    public override func uid() -> String {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ImageTarget_uid(cdata_, _return_value_)
        return String_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_String__dtor(p) }))
    }
    public override func name() -> String {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ImageTarget_name(cdata_, _return_value_)
        return String_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_String__dtor(p) }))
    }
    public override func setName(_ name: String) -> Void {
        easyar_ImageTarget_setName(cdata_, String_to_c(name).ptr_)
    }
    public override func meta() -> String {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ImageTarget_meta(cdata_, _return_value_)
        return String_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_String__dtor(p) }))
    }
    public override func setMeta(_ data: String) -> Void {
        easyar_ImageTarget_setMeta(cdata_, String_to_c(data).ptr_)
    }
}

public struct ImageTrackerMode : OptionSet {
    public let rawValue: Int32
    public init(rawValue: Int32) { self.rawValue = rawValue }

    public static let PreferQuality = ImageTrackerMode(rawValue: 0)
    public static let PreferPerformance = ImageTrackerMode(rawValue: 1)
}

public class ImageTrackerResult : TargetTrackerResult {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public override func targetInstances() -> [TargetInstance] {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ImageTrackerResult_targetInstances(cdata_, _return_value_)
        return ListOfTargetInstance_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_ListOfTargetInstance__dtor(p) }))
    }
    public override func setTargetInstances(_ instances: [TargetInstance]) -> Void {
        easyar_ImageTrackerResult_setTargetInstances(cdata_, ListOfTargetInstance_to_c(instances).ptr_)
    }
}

public class ImageTracker : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public static func isAvailable() -> Bool {
        let _return_value_ = easyar_ImageTracker_isAvailable()
        return _return_value_
    }
    public func feedbackFrameSink() -> FeedbackFrameSink {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ImageTracker_feedbackFrameSink(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_FeedbackFrameSink__typeName) as FeedbackFrameSink)
    }
    public func outputFrameSource() -> OutputFrameSource {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ImageTracker_outputFrameSource(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_OutputFrameSource__typeName) as OutputFrameSource)
    }
    public static func create() -> ImageTracker {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ImageTracker_create(_return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_ImageTracker__typeName) as ImageTracker)
    }
    public static func createWithMode(_ trackMode: ImageTrackerMode) -> ImageTracker {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ImageTracker_createWithMode(trackMode.rawValue, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_ImageTracker__typeName) as ImageTracker)
    }
    public func start() -> Bool {
        let _return_value_ = easyar_ImageTracker_start(cdata_)
        return _return_value_
    }
    public func stop() -> Void {
        easyar_ImageTracker_stop(cdata_)
    }
    public func close() -> Void {
        easyar_ImageTracker_close(cdata_)
    }
    public func loadTarget(_ target: Target, _ callbackScheduler: CallbackScheduler, _ callback: @escaping (Target, Bool) -> Void) -> Void {
        easyar_ImageTracker_loadTarget(cdata_, target.cdata_, callbackScheduler.cdata_, FunctorOfVoidFromTargetAndBool_to_c(callback))
    }
    public func unloadTarget(_ target: Target, _ callbackScheduler: CallbackScheduler, _ callback: @escaping (Target, Bool) -> Void) -> Void {
        easyar_ImageTracker_unloadTarget(cdata_, target.cdata_, callbackScheduler.cdata_, FunctorOfVoidFromTargetAndBool_to_c(callback))
    }
    public func targets() -> [Target] {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ImageTracker_targets(cdata_, _return_value_)
        return ListOfTarget_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_ListOfTarget__dtor(p) }))
    }
    public func setSimultaneousNum(_ num: Int32) -> Bool {
        let _return_value_ = easyar_ImageTracker_setSimultaneousNum(cdata_, num)
        return _return_value_
    }
    public func simultaneousNum() -> Int32 {
        let _return_value_ = easyar_ImageTracker_simultaneousNum(cdata_)
        return _return_value_
    }
}

public class Recorder : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public static func isAvailable() -> Bool {
        let _return_value_ = easyar_Recorder_isAvailable()
        return _return_value_
    }
    public static func requestPermissions(_ callbackScheduler: CallbackScheduler, _ permissionCallback: ((PermissionStatus, String) -> Void)?) -> Void {
        easyar_Recorder_requestPermissions(callbackScheduler.cdata_, (permissionCallback.map({(p) in easyar_OptionalOfFunctorOfVoidFromPermissionStatusAndString(has_value: true, value: FunctorOfVoidFromPermissionStatusAndString_to_c(p))}) ?? easyar_OptionalOfFunctorOfVoidFromPermissionStatusAndString(has_value: false, value: defaultValue())))
    }
    public static func create(_ config: RecorderConfiguration, _ callbackScheduler: CallbackScheduler, _ statusCallback: ((RecordStatus, String) -> Void)?) -> Recorder {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_Recorder_create(config.cdata_, callbackScheduler.cdata_, (statusCallback.map({(p) in easyar_OptionalOfFunctorOfVoidFromRecordStatusAndString(has_value: true, value: FunctorOfVoidFromRecordStatusAndString_to_c(p))}) ?? easyar_OptionalOfFunctorOfVoidFromRecordStatusAndString(has_value: false, value: defaultValue())), _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_Recorder__typeName) as Recorder)
    }
    public func start() -> Void {
        easyar_Recorder_start(cdata_)
    }
    public func updateFrame(_ texture: TextureId, _ width: Int32, _ height: Int32) -> Void {
        easyar_Recorder_updateFrame(cdata_, texture.cdata_, width, height)
    }
    public func stop() -> Bool {
        let _return_value_ = easyar_Recorder_stop(cdata_)
        return _return_value_
    }
}

public struct RecordProfile : OptionSet {
    public let rawValue: Int32
    public init(rawValue: Int32) { self.rawValue = rawValue }

    public static let Quality_1080P_Low = RecordProfile(rawValue: 0x00000001)
    public static let Quality_1080P_Middle = RecordProfile(rawValue: 0x00000002)
    public static let Quality_1080P_High = RecordProfile(rawValue: 0x00000004)
    public static let Quality_720P_Low = RecordProfile(rawValue: 0x00000008)
    public static let Quality_720P_Middle = RecordProfile(rawValue: 0x00000010)
    public static let Quality_720P_High = RecordProfile(rawValue: 0x00000020)
    public static let Quality_480P_Low = RecordProfile(rawValue: 0x00000040)
    public static let Quality_480P_Middle = RecordProfile(rawValue: 0x00000080)
    public static let Quality_480P_High = RecordProfile(rawValue: 0x00000100)
    public static let Quality_Default = RecordProfile(rawValue: 0x00000010)
}

public struct RecordVideoSize : OptionSet {
    public let rawValue: Int32
    public init(rawValue: Int32) { self.rawValue = rawValue }

    public static let Vid1080p = RecordVideoSize(rawValue: 0x00000002)
    public static let Vid720p = RecordVideoSize(rawValue: 0x00000010)
    public static let Vid480p = RecordVideoSize(rawValue: 0x00000080)
}

public struct RecordZoomMode : OptionSet {
    public let rawValue: Int32
    public init(rawValue: Int32) { self.rawValue = rawValue }

    public static let NoZoomAndClip = RecordZoomMode(rawValue: 0x00000000)
    public static let ZoomInWithAllContent = RecordZoomMode(rawValue: 0x00000001)
}

public struct RecordVideoOrientation : OptionSet {
    public let rawValue: Int32
    public init(rawValue: Int32) { self.rawValue = rawValue }

    public static let Landscape = RecordVideoOrientation(rawValue: 0x00000000)
    public static let Portrait = RecordVideoOrientation(rawValue: 0x00000001)
}

public struct RecordStatus : OptionSet {
    public let rawValue: Int32
    public init(rawValue: Int32) { self.rawValue = rawValue }

    public static let OnStarted = RecordStatus(rawValue: 0x00000002)
    public static let OnStopped = RecordStatus(rawValue: 0x00000004)
    public static let FailedToStart = RecordStatus(rawValue: 0x00000202)
    public static let FileSucceeded = RecordStatus(rawValue: 0x00000400)
    public static let FileFailed = RecordStatus(rawValue: 0x00000401)
    public static let LogInfo = RecordStatus(rawValue: 0x00000800)
    public static let LogError = RecordStatus(rawValue: 0x00001000)
}

public class RecorderConfiguration : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public convenience init() {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_RecorderConfiguration__ctor(_return_value_)
        self.init(cdata: _return_value_.pointee, deleter: { ptr in easyar_RecorderConfiguration__dtor(ptr) })
    }
    public func setOutputFile(_ path: String) -> Void {
        easyar_RecorderConfiguration_setOutputFile(cdata_, String_to_c(path).ptr_)
    }
    public func setProfile(_ profile: RecordProfile) -> Bool {
        let _return_value_ = easyar_RecorderConfiguration_setProfile(cdata_, profile.rawValue)
        return _return_value_
    }
    public func setVideoSize(_ framesize: RecordVideoSize) -> Void {
        easyar_RecorderConfiguration_setVideoSize(cdata_, framesize.rawValue)
    }
    public func setVideoBitrate(_ bitrate: Int32) -> Void {
        easyar_RecorderConfiguration_setVideoBitrate(cdata_, bitrate)
    }
    public func setChannelCount(_ count: Int32) -> Void {
        easyar_RecorderConfiguration_setChannelCount(cdata_, count)
    }
    public func setAudioSampleRate(_ samplerate: Int32) -> Void {
        easyar_RecorderConfiguration_setAudioSampleRate(cdata_, samplerate)
    }
    public func setAudioBitrate(_ bitrate: Int32) -> Void {
        easyar_RecorderConfiguration_setAudioBitrate(cdata_, bitrate)
    }
    public func setVideoOrientation(_ mode: RecordVideoOrientation) -> Void {
        easyar_RecorderConfiguration_setVideoOrientation(cdata_, mode.rawValue)
    }
    public func setZoomMode(_ mode: RecordZoomMode) -> Void {
        easyar_RecorderConfiguration_setZoomMode(cdata_, mode.rawValue)
    }
}

public struct MapTrackingStatus : OptionSet {
    public let rawValue: Int32
    public init(rawValue: Int32) { self.rawValue = rawValue }

    public static let NotTracking = MapTrackingStatus(rawValue: 0)
    public static let Tracking = MapTrackingStatus(rawValue: 1)
}

public class SparseSpatialMapResult : FrameFilterResult {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public func getMapTrackingStatus() -> MapTrackingStatus {
        let _return_value_ = easyar_SparseSpatialMapResult_getMapTrackingStatus(cdata_)
        return MapTrackingStatus(rawValue: _return_value_)
    }
    public func getVioPose() -> Matrix44F {
        let _return_value_ = easyar_SparseSpatialMapResult_getVioPose(cdata_)
        return Matrix44F_from_c(_return_value_)
    }
    public func getMapPose() -> Matrix44F {
        let _return_value_ = easyar_SparseSpatialMapResult_getMapPose(cdata_)
        return Matrix44F_from_c(_return_value_)
    }
    public func getLocalizedStatus() -> Bool {
        let _return_value_ = easyar_SparseSpatialMapResult_getLocalizedStatus(cdata_)
        return _return_value_
    }
    public func getCurrentLocalizedMapID() -> String {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_SparseSpatialMapResult_getCurrentLocalizedMapID(cdata_, _return_value_)
        return String_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_String__dtor(p) }))
    }
}

public struct PlaneType : OptionSet {
    public let rawValue: Int32
    public init(rawValue: Int32) { self.rawValue = rawValue }

    public static let Horizontal = PlaneType(rawValue: 0)
    public static let Vertical = PlaneType(rawValue: 1)
}

public class PlaneData : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public convenience init() {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_PlaneData__ctor(_return_value_)
        self.init(cdata: _return_value_.pointee, deleter: { ptr in easyar_PlaneData__dtor(ptr) })
    }
    public func getType() -> PlaneType {
        let _return_value_ = easyar_PlaneData_getType(cdata_)
        return PlaneType(rawValue: _return_value_)
    }
    public func getPose() -> Matrix44F {
        let _return_value_ = easyar_PlaneData_getPose(cdata_)
        return Matrix44F_from_c(_return_value_)
    }
    public func getExtentX() -> Float {
        let _return_value_ = easyar_PlaneData_getExtentX(cdata_)
        return _return_value_
    }
    public func getExtentZ() -> Float {
        let _return_value_ = easyar_PlaneData_getExtentZ(cdata_)
        return _return_value_
    }
}

public struct LocalizeConfig : OptionSet {
    public let rawValue: Int32
    public init(rawValue: Int32) { self.rawValue = rawValue }

    public static let UntilSuccess = LocalizeConfig(rawValue: 0)
    public static let Keep = LocalizeConfig(rawValue: 1)
    public static let Once = LocalizeConfig(rawValue: 2)
    public static let KeepUpdate = LocalizeConfig(rawValue: 3)
}

public struct MapConfig : OptionSet {
    public let rawValue: Int32
    public init(rawValue: Int32) { self.rawValue = rawValue }

    public static let NotExtend = MapConfig(rawValue: 0)
    public static let Extend = MapConfig(rawValue: 1)
}

public class SparseSpatialMapConfig : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public convenience init() {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_SparseSpatialMapConfig__ctor(_return_value_)
        self.init(cdata: _return_value_.pointee, deleter: { ptr in easyar_SparseSpatialMapConfig__dtor(ptr) })
    }
    public func setLocalizeConfig(_ value: LocalizeConfig) -> Void {
        easyar_SparseSpatialMapConfig_setLocalizeConfig(cdata_, value.rawValue)
    }
    public func getLocalizeConfig() -> LocalizeConfig {
        let _return_value_ = easyar_SparseSpatialMapConfig_getLocalizeConfig(cdata_)
        return LocalizeConfig(rawValue: _return_value_)
    }
    public func setMapConfig(_ value: MapConfig) -> Void {
        easyar_SparseSpatialMapConfig_setMapConfig(cdata_, value.rawValue)
    }
    public func getMapConfig() -> MapConfig {
        let _return_value_ = easyar_SparseSpatialMapConfig_getMapConfig(cdata_)
        return MapConfig(rawValue: _return_value_)
    }
}

public class SparseSpatialMap : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public static func isAvailable() -> Bool {
        let _return_value_ = easyar_SparseSpatialMap_isAvailable()
        return _return_value_
    }
    public func inputFrameSink() -> InputFrameSink {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_SparseSpatialMap_inputFrameSink(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_InputFrameSink__typeName) as InputFrameSink)
    }
    public func outputFrameSource() -> OutputFrameSource {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_SparseSpatialMap_outputFrameSource(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_OutputFrameSource__typeName) as OutputFrameSource)
    }
    public static func create() -> SparseSpatialMap {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_SparseSpatialMap_create(_return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_SparseSpatialMap__typeName) as SparseSpatialMap)
    }
    public func start() -> Bool {
        let _return_value_ = easyar_SparseSpatialMap_start(cdata_)
        return _return_value_
    }
    public func stop() -> Void {
        easyar_SparseSpatialMap_stop(cdata_)
    }
    public func close() -> Void {
        easyar_SparseSpatialMap_close(cdata_)
    }
    public func getPointCloudBuffer() -> Buffer {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_SparseSpatialMap_getPointCloudBuffer(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_Buffer__typeName) as Buffer)
    }
    public func getMapPlanes() -> [PlaneData] {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_SparseSpatialMap_getMapPlanes(cdata_, _return_value_)
        return ListOfPlaneData_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_ListOfPlaneData__dtor(p) }))
    }
    public func hitTestAgainstPointCloud(_ cameraImagePoint: Vec2F) -> [Vec3F] {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_SparseSpatialMap_hitTestAgainstPointCloud(cdata_, Vec2F_to_c(cameraImagePoint), _return_value_)
        return ListOfVec3F_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_ListOfVec3F__dtor(p) }))
    }
    public func hitTestAgainstPlanes(_ cameraImagePoint: Vec2F) -> [Vec3F] {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_SparseSpatialMap_hitTestAgainstPlanes(cdata_, Vec2F_to_c(cameraImagePoint), _return_value_)
        return ListOfVec3F_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_ListOfVec3F__dtor(p) }))
    }
    public func saveMap(_ callbackScheduler: CallbackScheduler, _ resultCallBack: @escaping (Buffer?) -> Void) -> Bool {
        let _return_value_ = easyar_SparseSpatialMap_saveMap(cdata_, callbackScheduler.cdata_, FunctorOfVoidFromOptionalOfBuffer_to_c(resultCallBack))
        return _return_value_
    }
    public func loadMap(_ pMapData: Buffer, _ mapID: String, _ callbackScheduler: CallbackScheduler, _ resultCallBack: ((Bool) -> Void)?) -> Bool {
        let _return_value_ = easyar_SparseSpatialMap_loadMap(cdata_, pMapData.cdata_, String_to_c(mapID).ptr_, callbackScheduler.cdata_, (resultCallBack.map({(p) in easyar_OptionalOfFunctorOfVoidFromBool(has_value: true, value: FunctorOfVoidFromBool_to_c(p))}) ?? easyar_OptionalOfFunctorOfVoidFromBool(has_value: false, value: defaultValue())))
        return _return_value_
    }
    public func unloadMap(_ mapID: String, _ callbackScheduler: CallbackScheduler, _ resultCallBack: ((Bool) -> Void)?) -> Bool {
        let _return_value_ = easyar_SparseSpatialMap_unloadMap(cdata_, String_to_c(mapID).ptr_, callbackScheduler.cdata_, (resultCallBack.map({(p) in easyar_OptionalOfFunctorOfVoidFromBool(has_value: true, value: FunctorOfVoidFromBool_to_c(p))}) ?? easyar_OptionalOfFunctorOfVoidFromBool(has_value: false, value: defaultValue())))
        return _return_value_
    }
    public func setConfig(_ config: SparseSpatialMapConfig) -> Void {
        easyar_SparseSpatialMap_setConfig(cdata_, config.cdata_)
    }
    public func getConfig() -> SparseSpatialMapConfig {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_SparseSpatialMap_getConfig(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_SparseSpatialMapConfig__typeName) as SparseSpatialMapConfig)
    }
    public func startLocalize() -> Bool {
        let _return_value_ = easyar_SparseSpatialMap_startLocalize(cdata_)
        return _return_value_
    }
    public func stopLocalize() -> Bool {
        let _return_value_ = easyar_SparseSpatialMap_stopLocalize(cdata_)
        return _return_value_
    }
}

public class ImageHelper {
    public static func decode(_ buffer: Buffer) -> Image? {
        let _return_value_ = UnsafeMutablePointer<easyar_OptionalOfImage>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ImageHelper_decode(buffer.cdata_, _return_value_)
        return (_return_value_.pointee.has_value ? (Object_from_c(Optional.some(_return_value_.pointee.value!)!, easyar_Image__typeName) as Image) : nil)
    }
}

public class CallbackScheduler : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
}

public class DelayedCallbackScheduler : CallbackScheduler {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public convenience init() {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_DelayedCallbackScheduler__ctor(_return_value_)
        self.init(cdata: _return_value_.pointee, deleter: { ptr in easyar_DelayedCallbackScheduler__dtor(ptr) })
    }
    public func runOne() -> Bool {
        let _return_value_ = easyar_DelayedCallbackScheduler_runOne(cdata_)
        return _return_value_
    }
}

public class ImmediateCallbackScheduler : CallbackScheduler {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public static func getDefault() -> ImmediateCallbackScheduler {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_ImmediateCallbackScheduler_getDefault(_return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_ImmediateCallbackScheduler__typeName) as ImmediateCallbackScheduler)
    }
}

public struct CameraDeviceFocusMode : OptionSet {
    public let rawValue: Int32
    public init(rawValue: Int32) { self.rawValue = rawValue }

    public static let Normal = CameraDeviceFocusMode(rawValue: 0)
    public static let Continousauto = CameraDeviceFocusMode(rawValue: 2)
    public static let Infinity = CameraDeviceFocusMode(rawValue: 3)
    public static let Macro = CameraDeviceFocusMode(rawValue: 4)
    public static let Medium = CameraDeviceFocusMode(rawValue: 5)
}

public struct AndroidCameraApiType : OptionSet {
    public let rawValue: Int32
    public init(rawValue: Int32) { self.rawValue = rawValue }

    public static let Camera1 = AndroidCameraApiType(rawValue: 0)
    public static let Camera2 = AndroidCameraApiType(rawValue: 1)
}

public struct CameraDevicePresetProfile : OptionSet {
    public let rawValue: Int32
    public init(rawValue: Int32) { self.rawValue = rawValue }

    public static let Photo = CameraDevicePresetProfile(rawValue: 0)
    public static let High = CameraDevicePresetProfile(rawValue: 1)
    public static let Medium = CameraDevicePresetProfile(rawValue: 2)
    public static let Low = CameraDevicePresetProfile(rawValue: 3)
}

public struct CameraState : OptionSet {
    public let rawValue: Int32
    public init(rawValue: Int32) { self.rawValue = rawValue }

    public static let Unknown = CameraState(rawValue: 0x00000000)
    public static let Disconnected = CameraState(rawValue: 0x00000001)
    public static let Preempted = CameraState(rawValue: 0x00000002)
}

public class CameraDevice : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public convenience init() {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_CameraDevice__ctor(_return_value_)
        self.init(cdata: _return_value_.pointee, deleter: { ptr in easyar_CameraDevice__dtor(ptr) })
    }
    public static func isAvailable() -> Bool {
        let _return_value_ = easyar_CameraDevice_isAvailable()
        return _return_value_
    }
    public func androidCameraApiType() -> AndroidCameraApiType {
        let _return_value_ = easyar_CameraDevice_androidCameraApiType(cdata_)
        return AndroidCameraApiType(rawValue: _return_value_)
    }
    public func setAndroidCameraApiType(_ type: AndroidCameraApiType) -> Void {
        easyar_CameraDevice_setAndroidCameraApiType(cdata_, type.rawValue)
    }
    public func bufferCapacity() -> Int32 {
        let _return_value_ = easyar_CameraDevice_bufferCapacity(cdata_)
        return _return_value_
    }
    public func setBufferCapacity(_ capacity: Int32) -> Void {
        easyar_CameraDevice_setBufferCapacity(cdata_, capacity)
    }
    public func inputFrameSource() -> InputFrameSource {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_CameraDevice_inputFrameSource(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_InputFrameSource__typeName) as InputFrameSource)
    }
    public func setStateChangedCallback(_ callbackScheduler: CallbackScheduler, _ stateChangedCallback: ((CameraState) -> Void)?) -> Void {
        easyar_CameraDevice_setStateChangedCallback(cdata_, callbackScheduler.cdata_, (stateChangedCallback.map({(p) in easyar_OptionalOfFunctorOfVoidFromCameraState(has_value: true, value: FunctorOfVoidFromCameraState_to_c(p))}) ?? easyar_OptionalOfFunctorOfVoidFromCameraState(has_value: false, value: defaultValue())))
    }
    public static func requestPermissions(_ callbackScheduler: CallbackScheduler, _ permissionCallback: ((PermissionStatus, String) -> Void)?) -> Void {
        easyar_CameraDevice_requestPermissions(callbackScheduler.cdata_, (permissionCallback.map({(p) in easyar_OptionalOfFunctorOfVoidFromPermissionStatusAndString(has_value: true, value: FunctorOfVoidFromPermissionStatusAndString_to_c(p))}) ?? easyar_OptionalOfFunctorOfVoidFromPermissionStatusAndString(has_value: false, value: defaultValue())))
    }
    public func openWithIndex(_ cameraIndex: Int32) -> Bool {
        let _return_value_ = easyar_CameraDevice_openWithIndex(cdata_, cameraIndex)
        return _return_value_
    }
    public func openWithType(_ type: CameraDeviceType) -> Bool {
        let _return_value_ = easyar_CameraDevice_openWithType(cdata_, type.rawValue)
        return _return_value_
    }
    public func start() -> Bool {
        let _return_value_ = easyar_CameraDevice_start(cdata_)
        return _return_value_
    }
    public func stop() -> Void {
        easyar_CameraDevice_stop(cdata_)
    }
    public func close() -> Void {
        easyar_CameraDevice_close(cdata_)
    }
    public func type() -> CameraDeviceType {
        let _return_value_ = easyar_CameraDevice_type(cdata_)
        return CameraDeviceType(rawValue: _return_value_)
    }
    public func cameraParameters() -> CameraParameters {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_CameraDevice_cameraParameters(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_CameraParameters__typeName) as CameraParameters)
    }
    public func setCameraParameters(_ cameraParameters: CameraParameters) -> Void {
        easyar_CameraDevice_setCameraParameters(cdata_, cameraParameters.cdata_)
    }
    public func size() -> Vec2I {
        let _return_value_ = easyar_CameraDevice_size(cdata_)
        return Vec2I_from_c(_return_value_)
    }
    public func supportedSizeCount() -> Int32 {
        let _return_value_ = easyar_CameraDevice_supportedSizeCount(cdata_)
        return _return_value_
    }
    public func supportedSize(_ index: Int32) -> Vec2I {
        let _return_value_ = easyar_CameraDevice_supportedSize(cdata_, index)
        return Vec2I_from_c(_return_value_)
    }
    public func setSize(_ size: Vec2I) -> Bool {
        let _return_value_ = easyar_CameraDevice_setSize(cdata_, Vec2I_to_c(size))
        return _return_value_
    }
    public func supportedFrameRateRangeCount() -> Int32 {
        let _return_value_ = easyar_CameraDevice_supportedFrameRateRangeCount(cdata_)
        return _return_value_
    }
    public func supportedFrameRateRangeLower(_ index: Int32) -> Float {
        let _return_value_ = easyar_CameraDevice_supportedFrameRateRangeLower(cdata_, index)
        return _return_value_
    }
    public func supportedFrameRateRangeUpper(_ index: Int32) -> Float {
        let _return_value_ = easyar_CameraDevice_supportedFrameRateRangeUpper(cdata_, index)
        return _return_value_
    }
    public func frameRateRange() -> Int32 {
        let _return_value_ = easyar_CameraDevice_frameRateRange(cdata_)
        return _return_value_
    }
    public func setFrameRateRange(_ index: Int32) -> Bool {
        let _return_value_ = easyar_CameraDevice_setFrameRateRange(cdata_, index)
        return _return_value_
    }
    public func setFlashTorchMode(_ on: Bool) -> Bool {
        let _return_value_ = easyar_CameraDevice_setFlashTorchMode(cdata_, on)
        return _return_value_
    }
    public func setFocusMode(_ focusMode: CameraDeviceFocusMode) -> Bool {
        let _return_value_ = easyar_CameraDevice_setFocusMode(cdata_, focusMode.rawValue)
        return _return_value_
    }
    public func autoFocus() -> Bool {
        let _return_value_ = easyar_CameraDevice_autoFocus(cdata_)
        return _return_value_
    }
    public func setPresentProfile(_ profile: CameraDevicePresetProfile) -> Bool {
        let _return_value_ = easyar_CameraDevice_setPresentProfile(cdata_, profile.rawValue)
        return _return_value_
    }
}

public struct CameraDevicePreference : OptionSet {
    public let rawValue: Int32
    public init(rawValue: Int32) { self.rawValue = rawValue }

    public static let PreferObjectSensing = CameraDevicePreference(rawValue: 0)
    public static let PreferSurfaceTracking = CameraDevicePreference(rawValue: 1)
    public static let PreferMotionTracking = CameraDevicePreference(rawValue: 2)
}

public class CameraDeviceSelector {
    public static func createCameraDevice(_ preference: CameraDevicePreference) -> CameraDevice {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_CameraDeviceSelector_createCameraDevice(preference.rawValue, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_CameraDevice__typeName) as CameraDevice)
    }
}

public class SignalSink : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public func handle() -> Void {
        easyar_SignalSink_handle(cdata_)
    }
}

public class SignalSource : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public func setHandler(_ handler: (() -> Void)?) -> Void {
        easyar_SignalSource_setHandler(cdata_, (handler.map({(p) in easyar_OptionalOfFunctorOfVoid(has_value: true, value: FunctorOfVoid_to_c(p))}) ?? easyar_OptionalOfFunctorOfVoid(has_value: false, value: defaultValue())))
    }
    public func connect(_ sink: SignalSink) -> Void {
        easyar_SignalSource_connect(cdata_, sink.cdata_)
    }
    public func disconnect() -> Void {
        easyar_SignalSource_disconnect(cdata_)
    }
}

public class InputFrameSink : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public func handle(_ inputData: InputFrame) -> Void {
        easyar_InputFrameSink_handle(cdata_, inputData.cdata_)
    }
}

public class InputFrameSource : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public func setHandler(_ handler: ((InputFrame) -> Void)?) -> Void {
        easyar_InputFrameSource_setHandler(cdata_, (handler.map({(p) in easyar_OptionalOfFunctorOfVoidFromInputFrame(has_value: true, value: FunctorOfVoidFromInputFrame_to_c(p))}) ?? easyar_OptionalOfFunctorOfVoidFromInputFrame(has_value: false, value: defaultValue())))
    }
    public func connect(_ sink: InputFrameSink) -> Void {
        easyar_InputFrameSource_connect(cdata_, sink.cdata_)
    }
    public func disconnect() -> Void {
        easyar_InputFrameSource_disconnect(cdata_)
    }
}

public class OutputFrameSink : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public func handle(_ inputData: OutputFrame) -> Void {
        easyar_OutputFrameSink_handle(cdata_, inputData.cdata_)
    }
}

public class OutputFrameSource : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public func setHandler(_ handler: ((OutputFrame) -> Void)?) -> Void {
        easyar_OutputFrameSource_setHandler(cdata_, (handler.map({(p) in easyar_OptionalOfFunctorOfVoidFromOutputFrame(has_value: true, value: FunctorOfVoidFromOutputFrame_to_c(p))}) ?? easyar_OptionalOfFunctorOfVoidFromOutputFrame(has_value: false, value: defaultValue())))
    }
    public func connect(_ sink: OutputFrameSink) -> Void {
        easyar_OutputFrameSource_connect(cdata_, sink.cdata_)
    }
    public func disconnect() -> Void {
        easyar_OutputFrameSource_disconnect(cdata_)
    }
}

public class FeedbackFrameSink : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public func handle(_ inputData: FeedbackFrame) -> Void {
        easyar_FeedbackFrameSink_handle(cdata_, inputData.cdata_)
    }
}

public class FeedbackFrameSource : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public func setHandler(_ handler: ((FeedbackFrame) -> Void)?) -> Void {
        easyar_FeedbackFrameSource_setHandler(cdata_, (handler.map({(p) in easyar_OptionalOfFunctorOfVoidFromFeedbackFrame(has_value: true, value: FunctorOfVoidFromFeedbackFrame_to_c(p))}) ?? easyar_OptionalOfFunctorOfVoidFromFeedbackFrame(has_value: false, value: defaultValue())))
    }
    public func connect(_ sink: FeedbackFrameSink) -> Void {
        easyar_FeedbackFrameSource_connect(cdata_, sink.cdata_)
    }
    public func disconnect() -> Void {
        easyar_FeedbackFrameSource_disconnect(cdata_)
    }
}

public class InputFrameFork : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public func input() -> InputFrameSink {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_InputFrameFork_input(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_InputFrameSink__typeName) as InputFrameSink)
    }
    public func output(_ index: Int32) -> InputFrameSource {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_InputFrameFork_output(cdata_, index, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_InputFrameSource__typeName) as InputFrameSource)
    }
    public func outputCount() -> Int32 {
        let _return_value_ = easyar_InputFrameFork_outputCount(cdata_)
        return _return_value_
    }
    public static func create(_ outputCount: Int32) -> InputFrameFork {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_InputFrameFork_create(outputCount, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_InputFrameFork__typeName) as InputFrameFork)
    }
}

public class OutputFrameFork : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public func input() -> OutputFrameSink {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_OutputFrameFork_input(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_OutputFrameSink__typeName) as OutputFrameSink)
    }
    public func output(_ index: Int32) -> OutputFrameSource {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_OutputFrameFork_output(cdata_, index, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_OutputFrameSource__typeName) as OutputFrameSource)
    }
    public func outputCount() -> Int32 {
        let _return_value_ = easyar_OutputFrameFork_outputCount(cdata_)
        return _return_value_
    }
    public static func create(_ outputCount: Int32) -> OutputFrameFork {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_OutputFrameFork_create(outputCount, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_OutputFrameFork__typeName) as OutputFrameFork)
    }
}

public class OutputFrameJoin : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public func input(_ index: Int32) -> OutputFrameSink {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_OutputFrameJoin_input(cdata_, index, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_OutputFrameSink__typeName) as OutputFrameSink)
    }
    public func output() -> OutputFrameSource {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_OutputFrameJoin_output(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_OutputFrameSource__typeName) as OutputFrameSource)
    }
    public func inputCount() -> Int32 {
        let _return_value_ = easyar_OutputFrameJoin_inputCount(cdata_)
        return _return_value_
    }
    public static func create(_ inputCount: Int32) -> OutputFrameJoin {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_OutputFrameJoin_create(inputCount, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_OutputFrameJoin__typeName) as OutputFrameJoin)
    }
    public static func createWithJoiner(_ inputCount: Int32, _ joiner: @escaping ([OutputFrame]) -> OutputFrame) -> OutputFrameJoin {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_OutputFrameJoin_createWithJoiner(inputCount, FunctorOfOutputFrameFromListOfOutputFrame_to_c(joiner), _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_OutputFrameJoin__typeName) as OutputFrameJoin)
    }
}

public class FeedbackFrameFork : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public func input() -> FeedbackFrameSink {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_FeedbackFrameFork_input(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_FeedbackFrameSink__typeName) as FeedbackFrameSink)
    }
    public func output(_ index: Int32) -> FeedbackFrameSource {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_FeedbackFrameFork_output(cdata_, index, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_FeedbackFrameSource__typeName) as FeedbackFrameSource)
    }
    public func outputCount() -> Int32 {
        let _return_value_ = easyar_FeedbackFrameFork_outputCount(cdata_)
        return _return_value_
    }
    public static func create(_ outputCount: Int32) -> FeedbackFrameFork {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_FeedbackFrameFork_create(outputCount, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_FeedbackFrameFork__typeName) as FeedbackFrameFork)
    }
}

public class InputFrameThrottler : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public func input() -> InputFrameSink {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_InputFrameThrottler_input(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_InputFrameSink__typeName) as InputFrameSink)
    }
    public func output() -> InputFrameSource {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_InputFrameThrottler_output(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_InputFrameSource__typeName) as InputFrameSource)
    }
    public func signalInput() -> SignalSink {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_InputFrameThrottler_signalInput(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_SignalSink__typeName) as SignalSink)
    }
    public static func create() -> InputFrameThrottler {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_InputFrameThrottler_create(_return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_InputFrameThrottler__typeName) as InputFrameThrottler)
    }
}

public class OutputFrameBuffer : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public func input() -> OutputFrameSink {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_OutputFrameBuffer_input(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_OutputFrameSink__typeName) as OutputFrameSink)
    }
    public func signalOutput() -> SignalSource {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_OutputFrameBuffer_signalOutput(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_SignalSource__typeName) as SignalSource)
    }
    public func peek() -> OutputFrame? {
        let _return_value_ = UnsafeMutablePointer<easyar_OptionalOfOutputFrame>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_OutputFrameBuffer_peek(cdata_, _return_value_)
        return (_return_value_.pointee.has_value ? (Object_from_c(Optional.some(_return_value_.pointee.value!)!, easyar_OutputFrame__typeName) as OutputFrame) : nil)
    }
    public static func create() -> OutputFrameBuffer {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_OutputFrameBuffer_create(_return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_OutputFrameBuffer__typeName) as OutputFrameBuffer)
    }
}

public class InputFrameToOutputFrameAdapter : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public func input() -> InputFrameSink {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_InputFrameToOutputFrameAdapter_input(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_InputFrameSink__typeName) as InputFrameSink)
    }
    public func output() -> OutputFrameSource {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_InputFrameToOutputFrameAdapter_output(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_OutputFrameSource__typeName) as OutputFrameSource)
    }
    public static func create() -> InputFrameToOutputFrameAdapter {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_InputFrameToOutputFrameAdapter_create(_return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_InputFrameToOutputFrameAdapter__typeName) as InputFrameToOutputFrameAdapter)
    }
}

public class InputFrameToFeedbackFrameAdapter : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public func input() -> InputFrameSink {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_InputFrameToFeedbackFrameAdapter_input(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_InputFrameSink__typeName) as InputFrameSink)
    }
    public func sideInput() -> OutputFrameSink {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_InputFrameToFeedbackFrameAdapter_sideInput(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_OutputFrameSink__typeName) as OutputFrameSink)
    }
    public func output() -> FeedbackFrameSource {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_InputFrameToFeedbackFrameAdapter_output(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_FeedbackFrameSource__typeName) as FeedbackFrameSource)
    }
    public static func create() -> InputFrameToFeedbackFrameAdapter {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_InputFrameToFeedbackFrameAdapter_create(_return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_InputFrameToFeedbackFrameAdapter__typeName) as InputFrameToFeedbackFrameAdapter)
    }
}

public class Engine {
    public static func schemaHash() -> Int32 {
        let _return_value_ = easyar_Engine_schemaHash()
        return _return_value_
    }
    public static func initialize(_ key: String) -> Bool {
        if (easyar_Engine_schemaHash() != -1339788856) {
            fatalError("SchemaHashNotMatched");
        }
        let _return_value_ = easyar_Engine_initialize(String_to_c(key).ptr_)
        return _return_value_
    }
    public static func onPause() -> Void {
        easyar_Engine_onPause()
    }
    public static func onResume() -> Void {
        easyar_Engine_onResume()
    }
    public static func errorMessage() -> String {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_Engine_errorMessage(_return_value_)
        return String_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_String__dtor(p) }))
    }
    public static func versionString() -> String {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_Engine_versionString(_return_value_)
        return String_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_String__dtor(p) }))
    }
    public static func name() -> String {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_Engine_name(_return_value_)
        return String_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_String__dtor(p) }))
    }
}

public struct InputFrameTrackingStatus : OptionSet {
    public let rawValue: Int32
    public init(rawValue: Int32) { self.rawValue = rawValue }

    public static let NotTracking = InputFrameTrackingStatus(rawValue: 0)
    public static let Tracking = InputFrameTrackingStatus(rawValue: 1)
}

public class InputFrame : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public func index() -> Int32 {
        let _return_value_ = easyar_InputFrame_index(cdata_)
        return _return_value_
    }
    public func image() -> Image {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_InputFrame_image(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_Image__typeName) as Image)
    }
    public func hasCameraParameters() -> Bool {
        let _return_value_ = easyar_InputFrame_hasCameraParameters(cdata_)
        return _return_value_
    }
    public func cameraParameters() -> CameraParameters {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_InputFrame_cameraParameters(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_CameraParameters__typeName) as CameraParameters)
    }
    public func hasTemporalInformation() -> Bool {
        let _return_value_ = easyar_InputFrame_hasTemporalInformation(cdata_)
        return _return_value_
    }
    public func timestamp() -> Double {
        let _return_value_ = easyar_InputFrame_timestamp(cdata_)
        return _return_value_
    }
    public func hasSpatialInformation() -> Bool {
        let _return_value_ = easyar_InputFrame_hasSpatialInformation(cdata_)
        return _return_value_
    }
    public func cameraTransform() -> Matrix44F {
        let _return_value_ = easyar_InputFrame_cameraTransform(cdata_)
        return Matrix44F_from_c(_return_value_)
    }
    public func trackingStatus() -> InputFrameTrackingStatus {
        let _return_value_ = easyar_InputFrame_trackingStatus(cdata_)
        return InputFrameTrackingStatus(rawValue: _return_value_)
    }
    public static func create(_ image: Image, _ cameraParameters: CameraParameters, _ timestamp: Double, _ cameraTransform: Matrix44F, _ trackingStatus: InputFrameTrackingStatus) -> InputFrame {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_InputFrame_create(image.cdata_, cameraParameters.cdata_, timestamp, Matrix44F_to_c(cameraTransform), trackingStatus.rawValue, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_InputFrame__typeName) as InputFrame)
    }
    public static func createWithImageAndCameraParametersAndTemporal(_ image: Image, _ cameraParameters: CameraParameters, _ timestamp: Double) -> InputFrame {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_InputFrame_createWithImageAndCameraParametersAndTemporal(image.cdata_, cameraParameters.cdata_, timestamp, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_InputFrame__typeName) as InputFrame)
    }
    public static func createWithImageAndCameraParameters(_ image: Image, _ cameraParameters: CameraParameters) -> InputFrame {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_InputFrame_createWithImageAndCameraParameters(image.cdata_, cameraParameters.cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_InputFrame__typeName) as InputFrame)
    }
    public static func createWithImage(_ image: Image) -> InputFrame {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_InputFrame_createWithImage(image.cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_InputFrame__typeName) as InputFrame)
    }
}

public class FrameFilterResult : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
}

public class OutputFrame : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public convenience init(_ inputFrame: InputFrame, _ results: [FrameFilterResult?]) {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_OutputFrame__ctor(inputFrame.cdata_, ListOfOptionalOfFrameFilterResult_to_c(results).ptr_, _return_value_)
        self.init(cdata: _return_value_.pointee, deleter: { ptr in easyar_OutputFrame__dtor(ptr) })
    }
    public func index() -> Int32 {
        let _return_value_ = easyar_OutputFrame_index(cdata_)
        return _return_value_
    }
    public func inputFrame() -> InputFrame {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_OutputFrame_inputFrame(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_InputFrame__typeName) as InputFrame)
    }
    public func results() -> [FrameFilterResult?] {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_OutputFrame_results(cdata_, _return_value_)
        return ListOfOptionalOfFrameFilterResult_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_ListOfOptionalOfFrameFilterResult__dtor(p) }))
    }
}

public class FeedbackFrame : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public convenience init(_ inputFrame: InputFrame, _ previousOutputFrame: OutputFrame?) {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_FeedbackFrame__ctor(inputFrame.cdata_, (previousOutputFrame.map({(p) in easyar_OptionalOfOutputFrame(has_value: true, value: p.cdata_)}) ?? easyar_OptionalOfOutputFrame(has_value: false, value: defaultValue())), _return_value_)
        self.init(cdata: _return_value_.pointee, deleter: { ptr in easyar_FeedbackFrame__dtor(ptr) })
    }
    public func inputFrame() -> InputFrame {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_FeedbackFrame_inputFrame(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_InputFrame__typeName) as InputFrame)
    }
    public func previousOutputFrame() -> OutputFrame? {
        let _return_value_ = UnsafeMutablePointer<easyar_OptionalOfOutputFrame>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_FeedbackFrame_previousOutputFrame(cdata_, _return_value_)
        return (_return_value_.pointee.has_value ? (Object_from_c(Optional.some(_return_value_.pointee.value!)!, easyar_OutputFrame__typeName) as OutputFrame) : nil)
    }
}

public struct PermissionStatus : OptionSet {
    public let rawValue: Int32
    public init(rawValue: Int32) { self.rawValue = rawValue }

    public static let Granted = PermissionStatus(rawValue: 0x00000000)
    public static let Denied = PermissionStatus(rawValue: 0x00000001)
    public static let Error = PermissionStatus(rawValue: 0x00000002)
}

public struct StorageType : OptionSet {
    public let rawValue: Int32
    public init(rawValue: Int32) { self.rawValue = rawValue }

    public static let App = StorageType(rawValue: 0)
    public static let Assets = StorageType(rawValue: 1)
    public static let Absolute = StorageType(rawValue: 2)
}

public class Target : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public func runtimeID() -> Int32 {
        let _return_value_ = easyar_Target_runtimeID(cdata_)
        return _return_value_
    }
    public func uid() -> String {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_Target_uid(cdata_, _return_value_)
        return String_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_String__dtor(p) }))
    }
    public func name() -> String {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_Target_name(cdata_, _return_value_)
        return String_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_String__dtor(p) }))
    }
    public func setName(_ name: String) -> Void {
        easyar_Target_setName(cdata_, String_to_c(name).ptr_)
    }
    public func meta() -> String {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_Target_meta(cdata_, _return_value_)
        return String_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_String__dtor(p) }))
    }
    public func setMeta(_ data: String) -> Void {
        easyar_Target_setMeta(cdata_, String_to_c(data).ptr_)
    }
}

public struct TargetStatus : OptionSet {
    public let rawValue: Int32
    public init(rawValue: Int32) { self.rawValue = rawValue }

    public static let Unknown = TargetStatus(rawValue: 0)
    public static let Undefined = TargetStatus(rawValue: 1)
    public static let Detected = TargetStatus(rawValue: 2)
    public static let Tracked = TargetStatus(rawValue: 3)
}

public class TargetInstance : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public convenience init() {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_TargetInstance__ctor(_return_value_)
        self.init(cdata: _return_value_.pointee, deleter: { ptr in easyar_TargetInstance__dtor(ptr) })
    }
    public func status() -> TargetStatus {
        let _return_value_ = easyar_TargetInstance_status(cdata_)
        return TargetStatus(rawValue: _return_value_)
    }
    public func target() -> Target? {
        let _return_value_ = UnsafeMutablePointer<easyar_OptionalOfTarget>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_TargetInstance_target(cdata_, _return_value_)
        return (_return_value_.pointee.has_value ? (Object_from_c(Optional.some(_return_value_.pointee.value!)!, easyar_Target__typeName) as Target) : nil)
    }
    public func pose() -> Matrix44F {
        let _return_value_ = easyar_TargetInstance_pose(cdata_)
        return Matrix44F_from_c(_return_value_)
    }
}

public class TargetTrackerResult : FrameFilterResult {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public func targetInstances() -> [TargetInstance] {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_TargetTrackerResult_targetInstances(cdata_, _return_value_)
        return ListOfTargetInstance_from_c(OpaquePointerContainer(Optional.some(_return_value_.pointee)!, { p in easyar_ListOfTargetInstance__dtor(p) }))
    }
    public func setTargetInstances(_ instances: [TargetInstance]) -> Void {
        easyar_TargetTrackerResult_setTargetInstances(cdata_, ListOfTargetInstance_to_c(instances).ptr_)
    }
}

public class TextureId : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public func getInt() -> Int32 {
        let _return_value_ = easyar_TextureId_getInt(cdata_)
        return _return_value_
    }
    public func getPointer() -> OpaquePointer? {
        let _return_value_ = easyar_TextureId_getPointer(cdata_)
        return _return_value_
    }
    public static func fromInt(_ value: Int32) -> TextureId {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_TextureId_fromInt(value, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_TextureId__typeName) as TextureId)
    }
    public static func fromPointer(_ ptr: OpaquePointer?) -> TextureId {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_TextureId_fromPointer(ptr, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_TextureId__typeName) as TextureId)
    }
}

public struct VideoStatus : OptionSet {
    public let rawValue: Int32
    public init(rawValue: Int32) { self.rawValue = rawValue }

    public static let Error = VideoStatus(rawValue: -1)
    public static let Ready = VideoStatus(rawValue: 0)
    public static let Completed = VideoStatus(rawValue: 1)
}

public struct VideoType : OptionSet {
    public let rawValue: Int32
    public init(rawValue: Int32) { self.rawValue = rawValue }

    public static let Normal = VideoType(rawValue: 0)
    public static let TransparentSideBySide = VideoType(rawValue: 1)
    public static let TransparentTopAndBottom = VideoType(rawValue: 2)
}

public class VideoPlayer : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public convenience init() {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_VideoPlayer__ctor(_return_value_)
        self.init(cdata: _return_value_.pointee, deleter: { ptr in easyar_VideoPlayer__dtor(ptr) })
    }
    public func setVideoType(_ videoType: VideoType) -> Void {
        easyar_VideoPlayer_setVideoType(cdata_, videoType.rawValue)
    }
    public func setRenderTexture(_ texture: TextureId) -> Void {
        easyar_VideoPlayer_setRenderTexture(cdata_, texture.cdata_)
    }
    public func `open`(_ path: String, _ storageType: StorageType, _ callbackScheduler: CallbackScheduler, _ callback: ((VideoStatus) -> Void)?) -> Void {
        easyar_VideoPlayer_open(cdata_, String_to_c(path).ptr_, storageType.rawValue, callbackScheduler.cdata_, (callback.map({(p) in easyar_OptionalOfFunctorOfVoidFromVideoStatus(has_value: true, value: FunctorOfVoidFromVideoStatus_to_c(p))}) ?? easyar_OptionalOfFunctorOfVoidFromVideoStatus(has_value: false, value: defaultValue())))
    }
    public func close() -> Void {
        easyar_VideoPlayer_close(cdata_)
    }
    public func play() -> Bool {
        let _return_value_ = easyar_VideoPlayer_play(cdata_)
        return _return_value_
    }
    public func stop() -> Void {
        easyar_VideoPlayer_stop(cdata_)
    }
    public func pause() -> Void {
        easyar_VideoPlayer_pause(cdata_)
    }
    public func isRenderTextureAvailable() -> Bool {
        let _return_value_ = easyar_VideoPlayer_isRenderTextureAvailable(cdata_)
        return _return_value_
    }
    public func updateFrame() -> Void {
        easyar_VideoPlayer_updateFrame(cdata_)
    }
    public func duration() -> Int32 {
        let _return_value_ = easyar_VideoPlayer_duration(cdata_)
        return _return_value_
    }
    public func currentPosition() -> Int32 {
        let _return_value_ = easyar_VideoPlayer_currentPosition(cdata_)
        return _return_value_
    }
    public func seek(_ position: Int32) -> Bool {
        let _return_value_ = easyar_VideoPlayer_seek(cdata_, position)
        return _return_value_
    }
    public func size() -> Vec2I {
        let _return_value_ = easyar_VideoPlayer_size(cdata_)
        return Vec2I_from_c(_return_value_)
    }
    public func volume() -> Float {
        let _return_value_ = easyar_VideoPlayer_volume(cdata_)
        return _return_value_
    }
    public func setVolume(_ volume: Float) -> Bool {
        let _return_value_ = easyar_VideoPlayer_setVolume(cdata_, volume)
        return _return_value_
    }
}

public class Buffer : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public static func wrap(_ ptr: OpaquePointer?, _ size: Int32, _ deleter: @escaping () -> Void) -> Buffer {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_Buffer_wrap(ptr, size, FunctorOfVoid_to_c(deleter), _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_Buffer__typeName) as Buffer)
    }
    public static func create(_ size: Int32) -> Buffer {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_Buffer_create(size, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_Buffer__typeName) as Buffer)
    }
    public func data() -> OpaquePointer? {
        let _return_value_ = easyar_Buffer_data(cdata_)
        return _return_value_
    }
    public func size() -> Int32 {
        let _return_value_ = easyar_Buffer_size(cdata_)
        return _return_value_
    }
    public static func memoryCopy(_ src: OpaquePointer?, _ dest: OpaquePointer?, _ length: Int32) -> Void {
        easyar_Buffer_memoryCopy(src, dest, length)
    }
    public func tryCopyFrom(_ src: OpaquePointer?, _ srcIndex: Int32, _ index: Int32, _ length: Int32) -> Bool {
        let _return_value_ = easyar_Buffer_tryCopyFrom(cdata_, src, srcIndex, index, length)
        return _return_value_
    }
    public func tryCopyTo(_ index: Int32, _ dest: OpaquePointer?, _ destIndex: Int32, _ length: Int32) -> Bool {
        let _return_value_ = easyar_Buffer_tryCopyTo(cdata_, index, dest, destIndex, length)
        return _return_value_
    }
    public func partition(_ index: Int32, _ length: Int32) -> Buffer {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_Buffer_partition(cdata_, index, length, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_Buffer__typeName) as Buffer)
    }
    public func tryCopyFromByteArray(_ src: [UInt8]) -> Bool {
        return tryCopyFromByteArray(src, 0, 0, Int32(src.count))
    }
    public func tryCopyFromByteArray(_ src: [UInt8], _ srcIndex: Int32, _ index: Int32, _ length: Int32) -> Bool {
        let srcSize = src.count
        let destSize = size()
        if (length < 0) || (srcIndex < 0) || (srcIndex > srcSize) || (srcIndex + length > srcSize) || (index < 0) || (index > destSize) || (index + length > destSize) {
            return false
        }
        if length == 0 { return true }
        var result = false
        src.withUnsafeBytes { p -> Void in
            result = tryCopyFrom(OpaquePointer(p.baseAddress!), srcIndex, index, length)
        }
        return result
    }
    public func tryCopyToByteArray(_ dest: [UInt8]) -> Bool {
        return tryCopyToByteArray(0, dest, 0, Int32(dest.count))
    }
    public func tryCopyToByteArray(_ index: Int32, _ dest: [UInt8], _ destIndex: Int32, _ length: Int32) -> Bool {
        let srcSize = size()
        let destSize = dest.count
        if (length < 0) || (index < 0) || (index > srcSize) || (index + length > srcSize) || (destIndex < 0) || (destIndex > destSize) || (destIndex + length > destSize) {
            return false
        }
        if length == 0 { return true }
        var result = false
        dest.withUnsafeBytes { p -> Void in
            result = tryCopyTo(0, OpaquePointer(p.baseAddress!), destIndex, length)
        }
        return result
    }
}

public class BufferDictionary : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public convenience init() {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_BufferDictionary__ctor(_return_value_)
        self.init(cdata: _return_value_.pointee, deleter: { ptr in easyar_BufferDictionary__dtor(ptr) })
    }
    public func count() -> Int32 {
        let _return_value_ = easyar_BufferDictionary_count(cdata_)
        return _return_value_
    }
    public func contains(_ path: String) -> Bool {
        let _return_value_ = easyar_BufferDictionary_contains(cdata_, String_to_c(path).ptr_)
        return _return_value_
    }
    public func tryGet(_ path: String) -> Buffer? {
        let _return_value_ = UnsafeMutablePointer<easyar_OptionalOfBuffer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_BufferDictionary_tryGet(cdata_, String_to_c(path).ptr_, _return_value_)
        return (_return_value_.pointee.has_value ? (Object_from_c(Optional.some(_return_value_.pointee.value!)!, easyar_Buffer__typeName) as Buffer) : nil)
    }
    public func set(_ path: String, _ buffer: Buffer) -> Void {
        easyar_BufferDictionary_set(cdata_, String_to_c(path).ptr_, buffer.cdata_)
    }
    public func remove(_ path: String) -> Bool {
        let _return_value_ = easyar_BufferDictionary_remove(cdata_, String_to_c(path).ptr_)
        return _return_value_
    }
    public func clear() -> Void {
        easyar_BufferDictionary_clear(cdata_)
    }
}

public class BufferPool : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public convenience init(_ block_size: Int32, _ capacity: Int32) {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_BufferPool__ctor(block_size, capacity, _return_value_)
        self.init(cdata: _return_value_.pointee, deleter: { ptr in easyar_BufferPool__dtor(ptr) })
    }
    public func block_size() -> Int32 {
        let _return_value_ = easyar_BufferPool_block_size(cdata_)
        return _return_value_
    }
    public func capacity() -> Int32 {
        let _return_value_ = easyar_BufferPool_capacity(cdata_)
        return _return_value_
    }
    public func size() -> Int32 {
        let _return_value_ = easyar_BufferPool_size(cdata_)
        return _return_value_
    }
    public func tryAcquire() -> Buffer? {
        let _return_value_ = UnsafeMutablePointer<easyar_OptionalOfBuffer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_BufferPool_tryAcquire(cdata_, _return_value_)
        return (_return_value_.pointee.has_value ? (Object_from_c(Optional.some(_return_value_.pointee.value!)!, easyar_Buffer__typeName) as Buffer) : nil)
    }
}

public struct CameraDeviceType : OptionSet {
    public let rawValue: Int32
    public init(rawValue: Int32) { self.rawValue = rawValue }

    public static let Default = CameraDeviceType(rawValue: 0)
    public static let Back = CameraDeviceType(rawValue: 1)
    public static let Front = CameraDeviceType(rawValue: 2)
}

public class CameraParameters : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public convenience init(_ size: Vec2I, _ focalLength: Vec2F, _ principalPoint: Vec2F, _ cameraDeviceType: CameraDeviceType, _ cameraOrientation: Int32) {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_CameraParameters__ctor(Vec2I_to_c(size), Vec2F_to_c(focalLength), Vec2F_to_c(principalPoint), cameraDeviceType.rawValue, cameraOrientation, _return_value_)
        self.init(cdata: _return_value_.pointee, deleter: { ptr in easyar_CameraParameters__dtor(ptr) })
    }
    public func size() -> Vec2I {
        let _return_value_ = easyar_CameraParameters_size(cdata_)
        return Vec2I_from_c(_return_value_)
    }
    public func focalLength() -> Vec2F {
        let _return_value_ = easyar_CameraParameters_focalLength(cdata_)
        return Vec2F_from_c(_return_value_)
    }
    public func principalPoint() -> Vec2F {
        let _return_value_ = easyar_CameraParameters_principalPoint(cdata_)
        return Vec2F_from_c(_return_value_)
    }
    public func cameraDeviceType() -> CameraDeviceType {
        let _return_value_ = easyar_CameraParameters_cameraDeviceType(cdata_)
        return CameraDeviceType(rawValue: _return_value_)
    }
    public func cameraOrientation() -> Int32 {
        let _return_value_ = easyar_CameraParameters_cameraOrientation(cdata_)
        return _return_value_
    }
    public static func createWithDefaultIntrinsics(_ size: Vec2I, _ cameraDeviceType: CameraDeviceType, _ cameraOrientation: Int32) -> CameraParameters {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_CameraParameters_createWithDefaultIntrinsics(Vec2I_to_c(size), cameraDeviceType.rawValue, cameraOrientation, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_CameraParameters__typeName) as CameraParameters)
    }
    public func imageOrientation(_ screenRotation: Int32) -> Int32 {
        let _return_value_ = easyar_CameraParameters_imageOrientation(cdata_, screenRotation)
        return _return_value_
    }
    public func imageHorizontalFlip(_ manualHorizontalFlip: Bool) -> Bool {
        let _return_value_ = easyar_CameraParameters_imageHorizontalFlip(cdata_, manualHorizontalFlip)
        return _return_value_
    }
    public func projection(_ nearPlane: Float, _ farPlane: Float, _ viewportAspectRatio: Float, _ screenRotation: Int32, _ combiningFlip: Bool, _ manualHorizontalFlip: Bool) -> Matrix44F {
        let _return_value_ = easyar_CameraParameters_projection(cdata_, nearPlane, farPlane, viewportAspectRatio, screenRotation, combiningFlip, manualHorizontalFlip)
        return Matrix44F_from_c(_return_value_)
    }
    public func imageProjection(_ viewportAspectRatio: Float, _ screenRotation: Int32, _ combiningFlip: Bool, _ manualHorizontalFlip: Bool) -> Matrix44F {
        let _return_value_ = easyar_CameraParameters_imageProjection(cdata_, viewportAspectRatio, screenRotation, combiningFlip, manualHorizontalFlip)
        return Matrix44F_from_c(_return_value_)
    }
    public func screenCoordinatesFromImageCoordinates(_ viewportAspectRatio: Float, _ screenRotation: Int32, _ combiningFlip: Bool, _ manualHorizontalFlip: Bool, _ imageCoordinates: Vec2F) -> Vec2F {
        let _return_value_ = easyar_CameraParameters_screenCoordinatesFromImageCoordinates(cdata_, viewportAspectRatio, screenRotation, combiningFlip, manualHorizontalFlip, Vec2F_to_c(imageCoordinates))
        return Vec2F_from_c(_return_value_)
    }
    public func imageCoordinatesFromScreenCoordinates(_ viewportAspectRatio: Float, _ screenRotation: Int32, _ combiningFlip: Bool, _ manualHorizontalFlip: Bool, _ screenCoordinates: Vec2F) -> Vec2F {
        let _return_value_ = easyar_CameraParameters_imageCoordinatesFromScreenCoordinates(cdata_, viewportAspectRatio, screenRotation, combiningFlip, manualHorizontalFlip, Vec2F_to_c(screenCoordinates))
        return Vec2F_from_c(_return_value_)
    }
    public func equalsTo(_ other: CameraParameters) -> Bool {
        let _return_value_ = easyar_CameraParameters_equalsTo(cdata_, other.cdata_)
        return _return_value_
    }
}

public struct PixelFormat : OptionSet {
    public let rawValue: Int32
    public init(rawValue: Int32) { self.rawValue = rawValue }

    public static let Unknown = PixelFormat(rawValue: 0)
    public static let Gray = PixelFormat(rawValue: 1)
    public static let YUV_NV21 = PixelFormat(rawValue: 2)
    public static let YUV_NV12 = PixelFormat(rawValue: 3)
    public static let YUV_I420 = PixelFormat(rawValue: 4)
    public static let YUV_YV12 = PixelFormat(rawValue: 5)
    public static let RGB888 = PixelFormat(rawValue: 6)
    public static let BGR888 = PixelFormat(rawValue: 7)
    public static let RGBA8888 = PixelFormat(rawValue: 8)
    public static let BGRA8888 = PixelFormat(rawValue: 9)
}

public class Image : RefBase {
    fileprivate override init(cdata: OpaquePointer, deleter: @escaping (OpaquePointer) -> Void) {
        super.init(cdata: cdata, deleter: deleter)
    }
    public convenience init(_ buffer: Buffer, _ format: PixelFormat, _ width: Int32, _ height: Int32) {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_Image__ctor(buffer.cdata_, format.rawValue, width, height, _return_value_)
        self.init(cdata: _return_value_.pointee, deleter: { ptr in easyar_Image__dtor(ptr) })
    }
    public func buffer() -> Buffer {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_Image_buffer(cdata_, _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_Buffer__typeName) as Buffer)
    }
    public func format() -> PixelFormat {
        let _return_value_ = easyar_Image_format(cdata_)
        return PixelFormat(rawValue: _return_value_)
    }
    public func width() -> Int32 {
        let _return_value_ = easyar_Image_width(cdata_)
        return _return_value_
    }
    public func height() -> Int32 {
        let _return_value_ = easyar_Image_height(cdata_)
        return _return_value_
    }
    public func empty() -> Bool {
        let _return_value_ = easyar_Image_empty(cdata_)
        return _return_value_
    }
}

public class JniUtility {
    public static func wrapByteArray(_ bytes: OpaquePointer?, _ readOnly: Bool, _ deleter: @escaping () -> Void) -> Buffer {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_JniUtility_wrapByteArray(bytes, readOnly, FunctorOfVoid_to_c(deleter), _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_Buffer__typeName) as Buffer)
    }
    public static func wrapBuffer(_ directBuffer: OpaquePointer?, _ deleter: @escaping () -> Void) -> Buffer {
        let _return_value_ = UnsafeMutablePointer<OpaquePointer>.allocate(capacity: 1)
        defer {
            _return_value_.deallocate()
        }
        easyar_JniUtility_wrapBuffer(directBuffer, FunctorOfVoid_to_c(deleter), _return_value_)
        return (Object_from_c(Optional.some(_return_value_.pointee)!, easyar_Buffer__typeName) as Buffer)
    }
}

public struct LogLevel : OptionSet {
    public let rawValue: Int32
    public init(rawValue: Int32) { self.rawValue = rawValue }

    public static let Error = LogLevel(rawValue: 0)
    public static let Warning = LogLevel(rawValue: 1)
    public static let Info = LogLevel(rawValue: 2)
}

public class Log {
    public static func setLogFunc(_ `func`: @escaping (LogLevel, String) -> Void) -> Void {
        easyar_Log_setLogFunc(FunctorOfVoidFromLogLevelAndString_to_c(`func`))
    }
    public static func resetLogFunc() -> Void {
        easyar_Log_resetLogFunc()
    }
}

public struct Matrix44F {
    public var data: (Float, Float, Float, Float, Float, Float, Float, Float, Float, Float, Float, Float, Float, Float, Float, Float)

    public init(_ data_0: Float, _ data_1: Float, _ data_2: Float, _ data_3: Float, _ data_4: Float, _ data_5: Float, _ data_6: Float, _ data_7: Float, _ data_8: Float, _ data_9: Float, _ data_10: Float, _ data_11: Float, _ data_12: Float, _ data_13: Float, _ data_14: Float, _ data_15: Float) {
        self.data = (data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, data_8, data_9, data_10, data_11, data_12, data_13, data_14, data_15);
    }
}

public struct Matrix33F {
    public var data: (Float, Float, Float, Float, Float, Float, Float, Float, Float)

    public init(_ data_0: Float, _ data_1: Float, _ data_2: Float, _ data_3: Float, _ data_4: Float, _ data_5: Float, _ data_6: Float, _ data_7: Float, _ data_8: Float) {
        self.data = (data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, data_8);
    }
}

public struct Vec4F {
    public var data: (Float, Float, Float, Float)

    public init(_ data_0: Float, _ data_1: Float, _ data_2: Float, _ data_3: Float) {
        self.data = (data_0, data_1, data_2, data_3);
    }
}

public struct Vec3F {
    public var data: (Float, Float, Float)

    public init(_ data_0: Float, _ data_1: Float, _ data_2: Float) {
        self.data = (data_0, data_1, data_2);
    }
}

public struct Vec2F {
    public var data: (Float, Float)

    public init(_ data_0: Float, _ data_1: Float) {
        self.data = (data_0, data_1);
    }
}

public struct Vec4I {
    public var data: (Int32, Int32, Int32, Int32)

    public init(_ data_0: Int32, _ data_1: Int32, _ data_2: Int32, _ data_3: Int32) {
        self.data = (data_0, data_1, data_2, data_3);
    }
}

public struct Vec2I {
    public var data: (Int32, Int32)

    public init(_ data_0: Int32, _ data_1: Int32) {
        self.data = (data_0, data_1);
    }
}
